@@ -1,19 +1,24 @@
 // This test works by setting up an exception for the tracker domain, which
 // disables all the anti-tracking tests.
 
-/* import-globals-from antitracking_head.js */
-
 add_task(async _ => {
-  Services.perms.add(
-    Services.io.newURI("https://tracking.example.org"),
+  PermissionTestUtils.add(
+    "https://tracking.example.org",
     "cookie",
     Services.perms.DENY_ACTION
   );
-  Services.perms.add(
-    Services.io.newURI("https://tracking.example.com"),
+  PermissionTestUtils.add(
+    "https://tracking.example.com",
     "cookie",
     Services.perms.DENY_ACTION
   );
+  // Grant interaction permission so we can directly call
+  // requestStorageAccess from the tracker.
+  PermissionTestUtils.add(
+    "https://tracking.example.org",
+    "storageAccessAPI",
+    Services.perms.ALLOW_ACTION
+  );
 
   registerCleanupFunction(_ => {
     Services.perms.removeAll();
@@ -28,6 +33,18 @@ AntiTracking._createTask({
   callback: async _ => {
     document.cookie = "name=value";
     ok(document.cookie == "", "All is blocked");
+
+    // requestStorageAccess should reject
+    SpecialPowers.wrap(document).notifyUserGestureActivation();
+    await document
+      .requestStorageAccess()
+      .then(() => {
+        ok(false, "Should not grant storage access");
+      })
+      .catch(() => {
+        ok(true, "Should not grant storage access");
+      });
+    SpecialPowers.wrap(document).clearUserGestureActivation();
   },
   extraPrefs: null,
   expectedBlockingNotifications:
