@@ -2,13 +2,9 @@
 /* vim: set sts=2 sw=2 et tw=80: */
 "use strict";
 
-const { Corroborate } = ChromeUtils.import(
-  "resource://gre/modules/Corroborate.jsm"
+const { Corroborate } = ChromeUtils.importESModule(
+  "resource://gre/modules/Corroborate.sys.mjs"
 );
-const { FileUtils } = ChromeUtils.import(
-  "resource://gre/modules/FileUtils.jsm"
-);
-const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 add_task(async function test_various_jars() {
   let result = await Corroborate.verifyJar(do_get_file("data/unsigned.xpi"));
@@ -33,19 +29,3 @@ add_task(async function test_various_jars() {
   );
   equal(result, true, "Components signed files do verify");
 });
-
-add_task(async function test_telemetry() {
-  Corroborate.reportTelemetry(true, false);
-
-  const scalars = Services.telemetry.getSnapshotForScalars("main", false)
-    .parent;
-
-  ok(
-    "corroborate.omnijar_corrupted" in scalars,
-    "omni jar should be corrupted"
-  );
-  ok(
-    "corroborate.system_addons_corrupted" in scalars,
-    "system addons should not be corrupted"
-  );
-});
