@@ -3,17 +3,23 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "MainThreadUtils.h"
 #include "mozilla/ExtensionPolicyService.h"
 #include "mozilla/extensions/DocumentObserver.h"
 #include "mozilla/extensions/WebExtensionContentScript.h"
 #include "mozilla/extensions/WebExtensionPolicy.h"
 
 #include "mozilla/AddonManagerWebAPI.h"
+#include "mozilla/BasePrincipal.h"
+#include "mozilla/dom/WindowGlobalChild.h"
 #include "mozilla/ResultExtensions.h"
 #include "mozilla/StaticPrefs_extensions.h"
+#include "nsContentUtils.h"
 #include "nsEscape.h"
+#include "nsGlobalWindowInner.h"
 #include "nsIObserver.h"
 #include "nsISubstitutingProtocolHandler.h"
+#include "nsLiteralString.h"
 #include "nsNetUtil.h"
 #include "nsPrintfCString.h"
 
@@ -24,6 +30,10 @@ using namespace dom;
 
 static const char kProto[] = "moz-extension";
 
+static const char kBackgroundScriptTypeDefault[] = "text/javascript";
+
+static const char kBackgroundScriptTypeModule[] = "module";
+
 static const char kBackgroundPageHTMLStart[] =
     "<!DOCTYPE html>\n\
 <html>\n\
@@ -32,15 +42,22 @@ static const char kBackgroundPageHTMLStart[] =
 
 static const char kBackgroundPageHTMLScript[] =
     "\n\
-    <script type=\"text/javascript\" src=\"%s\"></script>";
+    <script type=\"%s\" src=\"%s\"></script>";
 
 static const char kBackgroundPageHTMLEnd[] =
     "\n\
   </body>\n\
 </html>";
 
-static const char kRestrictedDomainPref[] =
-    "extensions.webextensions.restrictedDomains";
+#define BASE_CSP_PREF_V2 "extensions.webextensions.base-content-security-policy"
+#define DEFAULT_BASE_CSP_V2                                            \
+  "script-src 'self' https://* http://localhost:* http://127.0.0.1:* " \
+  "moz-extension: blob: filesystem: 'unsafe-eval' 'wasm-unsafe-eval' " \
+  "'unsafe-inline';"
+
+#define BASE_CSP_PREF_V3 \
+  "extensions.webextensions.base-content-security-policy.v3"
+#define DEFAULT_BASE_CSP_V3 "script-src 'self' 'wasm-unsafe-eval';"
 
 static inline ExtensionPolicyService& EPS() {
   return ExtensionPolicyService::GetSingleton();
@@ -65,14 +82,15 @@ static nsISubstitutingProtocolHandler* Proto() {
   return sHandler;
 }
 
-bool ParseGlobs(GlobalObject& aGlobal, Sequence<OwningMatchGlobOrString> aGlobs,
-                nsTArray<RefPtr<MatchGlob>>& aResult, ErrorResult& aRv) {
+bool ParseGlobs(GlobalObject& aGlobal,
+                Sequence<OwningMatchGlobOrUTF8String> aGlobs,
+                nsTArray<RefPtr<MatchGlobCore>>& aResult, ErrorResult& aRv) {
   for (auto& elem : aGlobs) {
     if (elem.IsMatchGlob()) {
-      aResult.AppendElement(elem.GetAsMatchGlob());
+      aResult.AppendElement(elem.GetAsMatchGlob()->Core());
     } else {
-      RefPtr<MatchGlob> glob =
-          MatchGlob::Constructor(aGlobal, elem.GetAsString(), true, aRv);
+      RefPtr<MatchGlobCore> glob =
+          new MatchGlobCore(elem.GetAsUTF8String(), true, aRv);
       if (aRv.Failed()) {
         return false;
       }
@@ -121,6 +139,149 @@ already_AddRefed<MatchPatternSet> ParseMatches(
   return result.forget();
 }
 
+WebAccessibleResource::WebAccessibleResource(
+    GlobalObject& aGlobal, const WebAccessibleResourceInit& aInit,
+    ErrorResult& aRv) {
+  ParseGlobs(aGlobal, aInit.mResources, mWebAccessiblePaths, aRv);
+  if (aRv.Failed()) {
+    return;
+  }
+
+  if (!aInit.mMatches.IsNull()) {
+    MatchPatternOptions options;
+    options.mRestrictSchemes = true;
+    RefPtr<MatchPatternSet> matches =
+        ParseMatches(aGlobal, aInit.mMatches.Value(), options,
+                     ErrorBehavior::CreateEmptyPattern, aRv);
+    MOZ_DIAGNOSTIC_ASSERT(!aRv.Failed());
+    mMatches = matches->Core();
+  }
+
+  if (!aInit.mExtension_ids.IsNull()) {
+    mExtensionIDs = new AtomSet(aInit.mExtension_ids.Value());
+  }
+}
+
+bool WebAccessibleResource::IsExtensionMatch(const URLInfo& aURI) {
+  if (!mExtensionIDs) {
+    return false;
+  }
+  RefPtr<WebExtensionPolicyCore> policy =
+      ExtensionPolicyService::GetCoreByHost(aURI.Host());
+  return policy && (mExtensionIDs->Contains(nsGkAtoms::_asterisk) ||
+                    mExtensionIDs->Contains(policy->Id()));
+}
+
+/*****************************************************************************
+ * WebExtensionPolicyCore
+ *****************************************************************************/
+
+WebExtensionPolicyCore::WebExtensionPolicyCore(GlobalObject& aGlobal,
+                                               WebExtensionPolicy* aPolicy,
+                                               const WebExtensionInit& aInit,
+                                               ErrorResult& aRv)
+    : mPolicy(aPolicy),
+      mId(NS_AtomizeMainThread(aInit.mId)),
+      mName(aInit.mName),
+      mType(NS_AtomizeMainThread(aInit.mType)),
+      mManifestVersion(aInit.mManifestVersion),
+      mExtensionPageCSP(aInit.mExtensionPageCSP),
+      mIsPrivileged(aInit.mIsPrivileged),
+      mIgnoreQuarantine(aInit.mIsPrivileged || aInit.mIgnoreQuarantine),
+      mTemporarilyInstalled(aInit.mTemporarilyInstalled),
+      mBackgroundWorkerScript(aInit.mBackgroundWorkerScript),
+      mPermissions(new AtomSet(aInit.mPermissions)) {
+  // In practice this is not necessary, but in tests where the uuid
+  // passed in is not lowercased various tests can fail.
+  ToLowerCase(aInit.mMozExtensionHostname, mHostname);
+
+  // Initialize the base CSP and extension page CSP
+  if (mManifestVersion < 3) {
+    nsresult rv = Preferences::GetString(BASE_CSP_PREF_V2, mBaseCSP);
+    if (NS_FAILED(rv)) {
+      mBaseCSP = NS_LITERAL_STRING_FROM_CSTRING(DEFAULT_BASE_CSP_V2);
+    }
+  } else {
+    nsresult rv = Preferences::GetString(BASE_CSP_PREF_V3, mBaseCSP);
+    if (NS_FAILED(rv)) {
+      mBaseCSP = NS_LITERAL_STRING_FROM_CSTRING(DEFAULT_BASE_CSP_V3);
+    }
+  }
+
+  if (mExtensionPageCSP.IsVoid()) {
+    if (mManifestVersion < 3) {
+      EPS().GetDefaultCSP(mExtensionPageCSP);
+    } else {
+      EPS().GetDefaultCSPV3(mExtensionPageCSP);
+    }
+  }
+
+  mWebAccessibleResources.SetCapacity(aInit.mWebAccessibleResources.Length());
+  for (const auto& resourceInit : aInit.mWebAccessibleResources) {
+    RefPtr<WebAccessibleResource> resource =
+        new WebAccessibleResource(aGlobal, resourceInit, aRv);
+    if (aRv.Failed()) {
+      return;
+    }
+    mWebAccessibleResources.AppendElement(std::move(resource));
+  }
+
+  nsresult rv = NS_NewURI(getter_AddRefs(mBaseURI), aInit.mBaseURL);
+  if (NS_FAILED(rv)) {
+    aRv.Throw(rv);
+  }
+}
+
+bool WebExtensionPolicyCore::SourceMayAccessPath(
+    const URLInfo& aURI, const nsACString& aPath) const {
+  if (aURI.Scheme() == nsGkAtoms::moz_extension &&
+      MozExtensionHostname().Equals(aURI.Host())) {
+    // An extension can always access it's own paths.
+    return true;
+  }
+  // Bug 1786564 Static themes need to allow access to theme resources.
+  if (Type() == nsGkAtoms::theme) {
+    RefPtr<WebExtensionPolicyCore> policyCore =
+        ExtensionPolicyService::GetCoreByHost(aURI.Host());
+    return policyCore != nullptr;
+  }
+
+  if (ManifestVersion() < 3) {
+    return IsWebAccessiblePath(aPath);
+  }
+  for (const auto& resource : mWebAccessibleResources) {
+    if (resource->SourceMayAccessPath(aURI, aPath)) {
+      return true;
+    }
+  }
+  return false;
+}
+
+bool WebExtensionPolicyCore::CanAccessURI(const URLInfo& aURI, bool aExplicit,
+                                          bool aCheckRestricted,
+                                          bool aAllowFilePermission) const {
+  if (aCheckRestricted && WebExtensionPolicy::IsRestrictedURI(aURI)) {
+    return false;
+  }
+  if (aCheckRestricted && QuarantinedFromURI(aURI)) {
+    return false;
+  }
+  if (!aAllowFilePermission && aURI.Scheme() == nsGkAtoms::file) {
+    return false;
+  }
+
+  AutoReadLock lock(mLock);
+  return mHostPermissions && mHostPermissions->Matches(aURI, aExplicit);
+}
+
+bool WebExtensionPolicyCore::QuarantinedFromDoc(const DocInfo& aDoc) const {
+  return QuarantinedFromURI(aDoc.PrincipalURL());
+}
+
+bool WebExtensionPolicyCore::QuarantinedFromURI(const URLInfo& aURI) const {
+  return !mIgnoreQuarantine && WebExtensionPolicy::IsQuarantinedURI(aURI);
+}
+
 /*****************************************************************************
  * WebExtensionPolicy
  *****************************************************************************/
@@ -128,44 +289,31 @@ already_AddRefed<MatchPatternSet> ParseMatches(
 WebExtensionPolicy::WebExtensionPolicy(GlobalObject& aGlobal,
                                        const WebExtensionInit& aInit,
                                        ErrorResult& aRv)
-    : mId(NS_AtomizeMainThread(aInit.mId)),
-      mHostname(aInit.mMozExtensionHostname),
-      mName(aInit.mName),
-      mExtensionPageCSP(aInit.mExtensionPageCSP),
-      mContentScriptCSP(aInit.mContentScriptCSP),
-      mLocalizeCallback(aInit.mLocalizeCallback),
-      mIsPrivileged(aInit.mIsPrivileged),
-      mPermissions(new AtomSet(aInit.mPermissions)) {
-  if (!ParseGlobs(aGlobal, aInit.mWebAccessibleResources, mWebAccessiblePaths,
-                  aRv)) {
+    : mCore(new WebExtensionPolicyCore(aGlobal, this, aInit, aRv)),
+      mLocalizeCallback(aInit.mLocalizeCallback) {
+  if (aRv.Failed()) {
     return;
   }
 
-  // We set this here to prevent this policy changing after creation.
-  mAllowPrivateBrowsingByDefault =
-      StaticPrefs::extensions_allowPrivateBrowsingByDefault();
-
   MatchPatternOptions options;
   options.mRestrictSchemes = !HasPermission(nsGkAtoms::mozillaAddons);
 
-  mHostPermissions = ParseMatches(aGlobal, aInit.mAllowedOrigins, options,
-                                  ErrorBehavior::CreateEmptyPattern, aRv);
+  // Set host permissions with SetAllowedOrigins to make sure the copy in core
+  // and WebExtensionPolicy stay in sync.
+  RefPtr<MatchPatternSet> hostPermissions =
+      ParseMatches(aGlobal, aInit.mAllowedOrigins, options,
+                   ErrorBehavior::CreateEmptyPattern, aRv);
   if (aRv.Failed()) {
     return;
   }
+  SetAllowedOrigins(*hostPermissions);
 
   if (!aInit.mBackgroundScripts.IsNull()) {
     mBackgroundScripts.SetValue().AppendElements(
         aInit.mBackgroundScripts.Value());
   }
 
-  if (mExtensionPageCSP.IsVoid()) {
-    EPS().GetDefaultCSP(mExtensionPageCSP);
-  }
-
-  if (mContentScriptCSP.IsVoid()) {
-    EPS().GetDefaultCSP(mContentScriptCSP);
-  }
+  mBackgroundTypeModule = aInit.mBackgroundTypeModule;
 
   mContentScripts.SetCapacity(aInit.mContentScripts.Length());
   for (const auto& scriptInit : aInit.mContentScripts) {
@@ -187,11 +335,6 @@ WebExtensionPolicy::WebExtensionPolicy(GlobalObject& aGlobal,
   if (aInit.mReadyPromise.WasPassed()) {
     mReadyPromise = &aInit.mReadyPromise.Value();
   }
-
-  nsresult rv = NS_NewURI(getter_AddRefs(mBaseURI), aInit.mBaseURL);
-  if (NS_FAILED(rv)) {
-    aRv.Throw(rv);
-  }
 }
 
 already_AddRefed<WebExtensionPolicy> WebExtensionPolicy::Constructor(
@@ -248,7 +391,13 @@ bool WebExtensionPolicy::Enable() {
     return false;
   }
 
-  Unused << Proto()->SetSubstitution(MozExtensionHostname(), mBaseURI);
+  if (XRE_IsParentProcess()) {
+    // Reserve a BrowsingContextGroup for use by this WebExtensionPolicy.
+    RefPtr<BrowsingContextGroup> group = BrowsingContextGroup::Create();
+    mBrowsingContextGroup = group->MakeKeepAlivePtr();
+  }
+
+  Unused << Proto()->SetSubstitution(MozExtensionHostname(), BaseURI());
 
   mActive = true;
   return true;
@@ -262,6 +411,12 @@ bool WebExtensionPolicy::Disable() {
     return false;
   }
 
+  if (XRE_IsParentProcess()) {
+    // Clear our BrowsingContextGroup reference. A new instance will be created
+    // when the extension is next activated.
+    mBrowsingContextGroup = nullptr;
+  }
+
   Unused << Proto()->SetSubstitution(MozExtensionHostname(), nullptr);
 
   mActive = false;
@@ -280,7 +435,7 @@ void WebExtensionPolicy::GetURL(const nsAString& aPath, nsAString& aResult,
 
 Result<nsString, nsresult> WebExtensionPolicy::GetURL(
     const nsAString& aPath) const {
-  nsPrintfCString spec("%s://%s/", kProto, mHostname.get());
+  nsPrintfCString spec("%s://%s/", kProto, MozExtensionHostname().get());
 
   nsCOMPtr<nsIURI> uri;
   MOZ_TRY(NS_NewURI(getter_AddRefs(uri), spec));
@@ -319,6 +474,15 @@ void WebExtensionPolicy::UnregisterContentScript(
   WebExtensionPolicy_Binding::ClearCachedContentScriptsValue(this);
 }
 
+void WebExtensionPolicy::SetAllowedOrigins(MatchPatternSet& aAllowedOrigins) {
+  // Make sure to keep the version in `WebExtensionPolicy` (which can be exposed
+  // back to script using AllowedOrigins()), and the version in
+  // `WebExtensionPolicyCore` (which is threadsafe) in sync.
+  AutoWriteLock lock(mCore->mLock);
+  mHostPermissions = &aAllowedOrigins;
+  mCore->mHostPermissions = aAllowedOrigins.Core();
+}
+
 void WebExtensionPolicy::InjectContentScripts(ErrorResult& aRv) {
   nsresult rv = EPS().InjectContentScripts(this);
   if (NS_FAILED(rv)) {
@@ -336,62 +500,16 @@ bool WebExtensionPolicy::IsExtensionProcess(GlobalObject& aGlobal) {
   return EPS().IsExtensionProcess();
 }
 
-namespace {
-/**
- * Maintains a dynamically updated AtomSet based on the comma-separated
- * values in the given string pref.
- */
-class AtomSetPref : public nsIObserver, public nsSupportsWeakReference {
- public:
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSIOBSERVER
-
-  static already_AddRefed<AtomSetPref> Create(const nsCString& aPref) {
-    RefPtr<AtomSetPref> self = new AtomSetPref(aPref.get());
-    Preferences::AddWeakObserver(self, aPref);
-    return self.forget();
-  }
-
-  const AtomSet& Get() const;
-
-  bool Contains(const nsAtom* aAtom) const { return Get().Contains(aAtom); }
-
- protected:
-  virtual ~AtomSetPref() = default;
-
-  explicit AtomSetPref(const char* aPref) : mPref(aPref) {}
-
- private:
-  mutable RefPtr<AtomSet> mAtomSet;
-  const char* mPref;
-};
-
-const AtomSet& AtomSetPref::Get() const {
-  if (!mAtomSet) {
-    nsAutoCString eltsString;
-    Unused << Preferences::GetCString(mPref, eltsString);
-
-    AutoTArray<nsString, 32> elts;
-    for (const nsACString& elt : eltsString.Split(',')) {
-      elts.AppendElement(NS_ConvertUTF8toUTF16(elt));
-      elts.LastElement().StripWhitespace();
-    }
-    mAtomSet = new AtomSet(elts);
-  }
-
-  return *mAtomSet;
+/* static */
+bool WebExtensionPolicy::BackgroundServiceWorkerEnabled(GlobalObject& aGlobal) {
+  return StaticPrefs::extensions_backgroundServiceWorker_enabled_AtStartup();
 }
 
-NS_IMETHODIMP
-AtomSetPref::Observe(nsISupports* aSubject, const char* aTopic,
-                     const char16_t* aData) {
-  mAtomSet = nullptr;
-  return NS_OK;
+/* static */
+bool WebExtensionPolicy::QuarantinedDomainsEnabled(GlobalObject& aGlobal) {
+  return EPS().GetQuarantinedDomainsEnabled();
 }
 
-NS_IMPL_ISUPPORTS(AtomSetPref, nsIObserver, nsISupportsWeakReference)
-};  // namespace
-
 /* static */
 bool WebExtensionPolicy::IsRestrictedDoc(const DocInfo& aDoc) {
   // With the exception of top-level about:blank documents with null
@@ -406,13 +524,10 @@ bool WebExtensionPolicy::IsRestrictedDoc(const DocInfo& aDoc) {
 
 /* static */
 bool WebExtensionPolicy::IsRestrictedURI(const URLInfo& aURI) {
-  static RefPtr<AtomSetPref> domains;
-  if (!domains) {
-    domains = AtomSetPref::Create(nsLiteralCString(kRestrictedDomainPref));
-    ClearOnShutdown(&domains);
-  }
+  RefPtr<AtomSet> restrictedDomains =
+      ExtensionPolicyService::RestrictedDomains();
 
-  if (domains->Contains(aURI.HostAtom())) {
+  if (restrictedDomains && restrictedDomains->Contains(aURI.HostAtom())) {
     return true;
   }
 
@@ -423,6 +538,22 @@ bool WebExtensionPolicy::IsRestrictedURI(const URLInfo& aURI) {
   return false;
 }
 
+/* static */
+bool WebExtensionPolicy::IsQuarantinedDoc(const DocInfo& aDoc) {
+  return IsQuarantinedURI(aDoc.PrincipalURL());
+}
+
+/* static */
+bool WebExtensionPolicy::IsQuarantinedURI(const URLInfo& aURI) {
+  // Ensure EPS is initialized before asking it about quarantined domains.
+  Unused << EPS();
+
+  RefPtr<AtomSet> quarantinedDomains =
+      ExtensionPolicyService::QuarantinedDomains();
+
+  return quarantinedDomains && quarantinedDomains->Contains(aURI.HostAtom());
+}
+
 nsCString WebExtensionPolicy::BackgroundPageHTML() const {
   nsCString result;
 
@@ -433,11 +564,13 @@ nsCString WebExtensionPolicy::BackgroundPageHTML() const {
 
   result.AppendLiteral(kBackgroundPageHTMLStart);
 
+  const char* scriptType = mBackgroundTypeModule ? kBackgroundScriptTypeModule
+                                                 : kBackgroundScriptTypeDefault;
+
   for (auto& script : mBackgroundScripts.Value()) {
     nsCString escaped;
     nsAppendEscapedHTML(NS_ConvertUTF16toUTF8(script), escaped);
-
-    result.AppendPrintf(kBackgroundPageHTMLScript, escaped.get());
+    result.AppendPrintf(kBackgroundPageHTMLScript, scriptType, escaped.get());
   }
 
   result.AppendLiteral(kBackgroundPageHTMLEnd);
@@ -451,7 +584,7 @@ void WebExtensionPolicy::Localize(const nsAString& aInput,
 }
 
 JSObject* WebExtensionPolicy::WrapObject(JSContext* aCx,
-                                         JS::HandleObject aGivenProto) {
+                                         JS::Handle<JSObject*> aGivenProto) {
   return WebExtensionPolicy_Binding::Wrap(aCx, this, aGivenProto);
 }
 
@@ -460,6 +593,10 @@ void WebExtensionPolicy::GetContentScripts(
   aScripts.AppendElements(mContentScripts);
 }
 
+bool WebExtensionPolicy::PrivateBrowsingAllowed() const {
+  return HasPermission(nsGkAtoms::privateBrowsingAllowedPermission);
+}
+
 bool WebExtensionPolicy::CanAccessContext(nsILoadContext* aContext) const {
   MOZ_ASSERT(aContext);
   return PrivateBrowsingAllowed() || !aContext->UsePrivateBrowsing();
@@ -477,7 +614,7 @@ bool WebExtensionPolicy::CanAccessWindow(
 }
 
 void WebExtensionPolicy::GetReadyPromise(
-    JSContext* aCx, JS::MutableHandleObject aResult) const {
+    JSContext* aCx, JS::MutableHandle<JSObject*> aResult) const {
   if (mReadyPromise) {
     aResult.set(mReadyPromise->PromiseObj());
   } else {
@@ -485,11 +622,41 @@ void WebExtensionPolicy::GetReadyPromise(
   }
 }
 
-NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_WEAK_PTR(WebExtensionPolicy, mParent,
-                                               mLocalizeCallback,
-                                               mHostPermissions,
-                                               mWebAccessiblePaths,
-                                               mContentScripts)
+uint64_t WebExtensionPolicy::GetBrowsingContextGroupId() const {
+  MOZ_ASSERT(XRE_IsParentProcess() && mActive);
+  return mBrowsingContextGroup ? mBrowsingContextGroup->Id() : 0;
+}
+
+uint64_t WebExtensionPolicy::GetBrowsingContextGroupId(ErrorResult& aRv) {
+  if (XRE_IsParentProcess() && mActive) {
+    return GetBrowsingContextGroupId();
+  }
+  aRv.ThrowInvalidAccessError(
+      "browsingContextGroupId only available for active policies in the "
+      "parent process");
+  return 0;
+}
+
+WebExtensionPolicy::~WebExtensionPolicy() { mCore->ClearPolicyWeakRef(); }
+
+NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(WebExtensionPolicy)
+NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(WebExtensionPolicy)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mParent)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mBrowsingContextGroup)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mLocalizeCallback)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mHostPermissions)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mContentScripts)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
+  AssertIsOnMainThread();
+  tmp->mCore->ClearPolicyWeakRef();
+NS_IMPL_CYCLE_COLLECTION_UNLINK_END
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(WebExtensionPolicy)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mParent)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mBrowsingContextGroup)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mLocalizeCallback)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mHostPermissions)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mContentScripts)
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(WebExtensionPolicy)
   NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
@@ -535,6 +702,7 @@ MozDocumentMatcher::MozDocumentMatcher(GlobalObject& aGlobal,
     : mHasActiveTabPermission(aInit.mHasActiveTabPermission),
       mRestricted(aRestricted),
       mAllFrames(aInit.mAllFrames),
+      mCheckPermissions(aInit.mCheckPermissions),
       mFrameID(aInit.mFrameID),
       mMatchAboutBlank(aInit.mMatchAboutBlank) {
   MatchPatternOptions options;
@@ -568,6 +736,17 @@ MozDocumentMatcher::MozDocumentMatcher(GlobalObject& aGlobal,
       return;
     }
   }
+
+  if (!aInit.mOriginAttributesPatterns.IsNull()) {
+    Sequence<OriginAttributesPattern>& arr =
+        mOriginAttributesPatterns.SetValue();
+    for (const auto& pattern : aInit.mOriginAttributesPatterns.Value()) {
+      if (!arr.AppendElement(OriginAttributesPattern(pattern), fallible)) {
+        aRv.Throw(NS_ERROR_OUT_OF_MEMORY);
+        return;
+      }
+    }
+  }
 }
 
 WebExtensionContentScript::WebExtensionContentScript(
@@ -580,9 +759,15 @@ WebExtensionContentScript::WebExtensionContentScript(
   mCssPaths.Assign(aInit.mCssPaths);
   mJsPaths.Assign(aInit.mJsPaths);
   mExtension = &aExtension;
+
+  // Origin permissions are optional in mv3, so always check them at runtime.
+  if (mExtension->ManifestVersion() >= 3) {
+    mCheckPermissions = true;
+  }
 }
 
-bool MozDocumentMatcher::Matches(const DocInfo& aDoc) const {
+bool MozDocumentMatcher::Matches(const DocInfo& aDoc,
+                                 bool aIgnorePermissions) const {
   if (!mFrameID.IsNull()) {
     if (aDoc.FrameID() != mFrameID.Value()) {
       return false;
@@ -599,6 +784,21 @@ bool MozDocumentMatcher::Matches(const DocInfo& aDoc) const {
     return false;
   }
 
+  if (loadContext && !mOriginAttributesPatterns.IsNull()) {
+    OriginAttributes docShellAttrs;
+    loadContext->GetOriginAttributes(docShellAttrs);
+    bool patternMatch = false;
+    for (const auto& pattern : mOriginAttributesPatterns.Value()) {
+      if (pattern.Matches(docShellAttrs)) {
+        patternMatch = true;
+        break;
+      }
+    }
+    if (!patternMatch) {
+      return false;
+    }
+  }
+
   if (!mMatchAboutBlank && aDoc.URL().InheritsPrincipal()) {
     return false;
   }
@@ -613,20 +813,35 @@ bool MozDocumentMatcher::Matches(const DocInfo& aDoc) const {
     return true;
   }
 
-  if (mRestricted && mExtension->IsRestrictedDoc(aDoc)) {
+  if (mRestricted && WebExtensionPolicy::IsRestrictedDoc(aDoc)) {
+    return false;
+  }
+
+  if (mRestricted && mExtension && mExtension->QuarantinedFromDoc(aDoc)) {
     return false;
   }
 
   auto& urlinfo = aDoc.PrincipalURL();
-  if (mHasActiveTabPermission && aDoc.ShouldMatchActiveTabPermission() &&
-      MatchPattern::MatchesAllURLs(urlinfo)) {
-    return true;
+  if (mExtension && mExtension->ManifestVersion() >= 3) {
+    // In MV3, activeTab only allows access to same-origin iframes.
+    if (mHasActiveTabPermission && aDoc.IsSameOriginWithTop() &&
+        MatchPattern::MatchesAllURLs(urlinfo)) {
+      return true;
+    }
+  } else {
+    if (mHasActiveTabPermission && aDoc.ShouldMatchActiveTabPermission() &&
+        MatchPattern::MatchesAllURLs(urlinfo)) {
+      return true;
+    }
   }
 
-  return MatchesURI(urlinfo);
+  return MatchesURI(urlinfo, aIgnorePermissions);
 }
 
-bool MozDocumentMatcher::MatchesURI(const URLInfo& aURL) const {
+bool MozDocumentMatcher::MatchesURI(const URLInfo& aURL,
+                                    bool aIgnorePermissions) const {
+  MOZ_ASSERT((!mRestricted && !mCheckPermissions) || mExtension);
+
   if (!mMatches->Matches(aURL)) {
     return false;
   }
@@ -635,34 +850,62 @@ bool MozDocumentMatcher::MatchesURI(const URLInfo& aURL) const {
     return false;
   }
 
-  if (!mIncludeGlobs.IsNull() && !mIncludeGlobs.Value().Matches(aURL.Spec())) {
+  if (!mIncludeGlobs.IsNull() && !mIncludeGlobs.Value().Matches(aURL.CSpec())) {
     return false;
   }
 
-  if (!mExcludeGlobs.IsNull() && mExcludeGlobs.Value().Matches(aURL.Spec())) {
+  if (!mExcludeGlobs.IsNull() && mExcludeGlobs.Value().Matches(aURL.CSpec())) {
     return false;
   }
 
-  if (mRestricted && mExtension->IsRestrictedURI(aURL)) {
+  if (mRestricted && WebExtensionPolicy::IsRestrictedURI(aURL)) {
+    return false;
+  }
+
+  if (mRestricted && mExtension->QuarantinedFromURI(aURL)) {
+    return false;
+  }
+
+  if (mCheckPermissions && !aIgnorePermissions &&
+      !mExtension->CanAccessURI(aURL, false, false, true)) {
     return false;
   }
 
   return true;
 }
 
+bool MozDocumentMatcher::MatchesWindowGlobal(WindowGlobalChild& aWindow,
+                                             bool aIgnorePermissions) const {
+  if (aWindow.IsClosed() || !aWindow.IsCurrentGlobal()) {
+    return false;
+  }
+  nsGlobalWindowInner* inner = aWindow.GetWindowGlobal();
+  if (!inner || !inner->GetDocShell()) {
+    return false;
+  }
+  return Matches(inner->GetOuterWindow(), aIgnorePermissions);
+}
+
+void MozDocumentMatcher::GetOriginAttributesPatterns(
+    JSContext* aCx, JS::MutableHandle<JS::Value> aVal,
+    ErrorResult& aError) const {
+  if (!ToJSValue(aCx, mOriginAttributesPatterns, aVal)) {
+    aError.NoteJSContextException(aCx);
+  }
+}
+
 JSObject* MozDocumentMatcher::WrapObject(JSContext* aCx,
-                                         JS::HandleObject aGivenProto) {
+                                         JS::Handle<JSObject*> aGivenProto) {
   return MozDocumentMatcher_Binding::Wrap(aCx, this, aGivenProto);
 }
 
-JSObject* WebExtensionContentScript::WrapObject(JSContext* aCx,
-                                                JS::HandleObject aGivenProto) {
+JSObject* WebExtensionContentScript::WrapObject(
+    JSContext* aCx, JS::Handle<JSObject*> aGivenProto) {
   return WebExtensionContentScript_Binding::Wrap(aCx, this, aGivenProto);
 }
 
 NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE(MozDocumentMatcher, mMatches,
-                                      mExcludeMatches, mIncludeGlobs,
-                                      mExcludeGlobs, mExtension)
+                                      mExcludeMatches, mExtension)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(MozDocumentMatcher)
   NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
@@ -718,7 +961,7 @@ void DocumentObserver::NotifyMatch(MozDocumentMatcher& aMatcher,
 }
 
 JSObject* DocumentObserver::WrapObject(JSContext* aCx,
-                                       JS::HandleObject aGivenProto) {
+                                       JS::Handle<JSObject*> aGivenProto) {
   return MozDocumentObserver_Binding::Wrap(aCx, this, aGivenProto);
 }
 
@@ -746,7 +989,9 @@ DocInfo::DocInfo(nsPIDOMWindowOuter* aWindow)
 bool DocInfo::IsTopLevel() const {
   if (mIsTopLevel.isNothing()) {
     struct Matcher {
-      bool operator()(Window aWin) { return aWin->IsTopLevelWindow(); }
+      bool operator()(Window aWin) {
+        return aWin->GetBrowsingContext()->IsTop();
+      }
       bool operator()(LoadInfo aLoadInfo) {
         return aLoadInfo->GetIsTopLevelLoad();
       }
@@ -757,33 +1002,19 @@ bool DocInfo::IsTopLevel() const {
 }
 
 bool WindowShouldMatchActiveTab(nsPIDOMWindowOuter* aWin) {
-  if (aWin->IsTopLevelWindow()) {
-    return true;
-  }
+  for (WindowContext* wc = aWin->GetCurrentInnerWindow()->GetWindowContext();
+       wc; wc = wc->GetParentWindowContext()) {
+    BrowsingContext* bc = wc->GetBrowsingContext();
+    if (bc->IsTopContent()) {
+      return true;
+    }
 
-  nsIDocShell* docshell = aWin->GetDocShell();
-  if (!docshell || docshell->GetCreatedDynamically()) {
-    return false;
+    if (bc->CreatedDynamically() || !wc->GetIsOriginalFrameSource()) {
+      return false;
+    }
   }
-
-  Document* doc = aWin->GetExtantDoc();
-  if (!doc) {
-    return false;
-  }
-
-  nsIChannel* channel = doc->GetChannel();
-  if (!channel) {
-    return false;
-  }
-
-  nsCOMPtr<nsILoadInfo> loadInfo = channel->LoadInfo();
-  if (!loadInfo->GetOriginalFrameSrcLoad()) {
-    return false;
-  }
-
-  nsCOMPtr<nsPIDOMWindowOuter> parent = aWin->GetInProcessParent();
-  MOZ_ASSERT(parent != nullptr);
-  return WindowShouldMatchActiveTab(parent);
+  MOZ_ASSERT_UNREACHABLE("Should reach top content before end of loop");
+  return false;
 }
 
 bool DocInfo::ShouldMatchActiveTabPermission() const {
@@ -794,15 +1025,28 @@ bool DocInfo::ShouldMatchActiveTabPermission() const {
   return mObj.match(Matcher());
 }
 
+bool DocInfo::IsSameOriginWithTop() const {
+  struct Matcher {
+    bool operator()(Window aWin) {
+      WindowContext* wc = aWin->GetCurrentInnerWindow()->GetWindowContext();
+      return wc && wc->SameOriginWithTop();
+    }
+    bool operator()(LoadInfo aLoadInfo) { return false; }
+  };
+  return mObj.match(Matcher());
+}
+
 uint64_t DocInfo::FrameID() const {
   if (mFrameID.isNothing()) {
     if (IsTopLevel()) {
       mFrameID.emplace(0);
     } else {
       struct Matcher {
-        uint64_t operator()(Window aWin) { return aWin->WindowID(); }
+        uint64_t operator()(Window aWin) {
+          return aWin->GetBrowsingContext()->Id();
+        }
         uint64_t operator()(LoadInfo aLoadInfo) {
-          return aLoadInfo->GetOuterWindowID();
+          return aLoadInfo->GetBrowsingContextID();
         }
       };
       mFrameID.emplace(mObj.match(Matcher()));
