@@ -2,8 +2,8 @@
 /* vim: set sts=2 sw=2 et tw=80: */
 "use strict";
 
-const { ExtensionParent } = ChromeUtils.import(
-  "resource://gre/modules/ExtensionParent.jsm"
+const { ExtensionParent } = ChromeUtils.importESModule(
+  "resource://gre/modules/ExtensionParent.sys.mjs"
 );
 
 const ENABLE_COUNTER_PREF =
@@ -44,9 +44,9 @@ async function test_counter() {
     });
 
     // getURL() is done in the child, let do three
-    browser.extension.getURL("beasts/frog.html");
-    browser.extension.getURL("beasts/frog2.html");
-    browser.extension.getURL("beasts/frog3.html");
+    browser.runtime.getURL("beasts/frog.html");
+    browser.runtime.getURL("beasts/frog2.html");
+    browser.runtime.getURL("beasts/frog3.html");
     browser.test.sendMessage("done");
   }
 
