@@ -6,6 +6,11 @@
 // from the JSONFile backend to the IDB backend.
 
 AddonTestUtils.init(this);
+
+// Create appInfo before importing any other jsm file, to prevent
+// Services.appinfo to be cached before an appInfo.version is
+// actually defined (which prevent failures to be triggered when
+// the test run in a non nightly build).
 AddonTestUtils.createAppInfo(
   "xpcshell@tests.mozilla.org",
   "XPCShell",
@@ -13,38 +18,37 @@ AddonTestUtils.createAppInfo(
   "42"
 );
 
-const { ExtensionStorage } = ChromeUtils.import(
-  "resource://gre/modules/ExtensionStorage.jsm"
+const { getTrimmedString } = ChromeUtils.importESModule(
+  "resource://gre/modules/ExtensionTelemetry.sys.mjs"
 );
-const { TelemetryController } = ChromeUtils.import(
-  "resource://gre/modules/TelemetryController.jsm"
+const { ExtensionStorage } = ChromeUtils.importESModule(
+  "resource://gre/modules/ExtensionStorage.sys.mjs"
 );
-
-const { ExtensionStorageIDB } = ChromeUtils.import(
-  "resource://gre/modules/ExtensionStorageIDB.jsm"
+const { ExtensionStorageIDB } = ChromeUtils.importESModule(
+  "resource://gre/modules/ExtensionStorageIDB.sys.mjs"
 );
-
-const { getTrimmedString } = ChromeUtils.import(
-  "resource://gre/modules/ExtensionTelemetry.jsm",
-  null
+const { TelemetryController } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryController.sys.mjs"
+);
+const { TelemetryTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TelemetryTestUtils.sys.mjs"
 );
-
-XPCOMUtils.defineLazyModuleGetters(this, {
-  OS: "resource://gre/modules/osfile.jsm",
-});
 
 const { promiseShutdownManager, promiseStartupManager } = AddonTestUtils;
 
-const {
-  IDB_MIGRATED_PREF_BRANCH,
-  IDB_MIGRATE_RESULT_HISTOGRAM,
-} = ExtensionStorageIDB;
+const { IDB_MIGRATED_PREF_BRANCH, IDB_MIGRATE_RESULT_HISTOGRAM } =
+  ExtensionStorageIDB;
 const CATEGORIES = ["success", "failure"];
 const EVENT_CATEGORY = "extensions.data";
 const EVENT_OBJECT = "storageLocal";
-const EVENT_METHODS = ["migrateResult"];
+const EVENT_METHOD = "migrateResult";
 const LEAVE_STORAGE_PREF = "extensions.webextensions.keepStorageOnUninstall";
 const LEAVE_UUID_PREF = "extensions.webextensions.keepUuidOnUninstall";
+const TELEMETRY_EVENTS_FILTER = {
+  category: "extensions.data",
+  method: "migrateResult",
+  object: "storageLocal",
+};
 
 async function createExtensionJSONFileWithData(extensionId, data) {
   await ExtensionStorage.set(extensionId, data);
@@ -52,7 +56,7 @@ async function createExtensionJSONFileWithData(extensionId, data) {
   await jsonFile._save();
   const oldStorageFilename = ExtensionStorage.getStorageFile(extensionId);
   equal(
-    await OS.File.exists(oldStorageFilename),
+    await IOUtils.exists(oldStorageFilename),
     true,
     "The old json file has been created"
   );
@@ -84,38 +88,15 @@ function assertMigrationHistogramCount(category, expectedCount) {
   );
 }
 
-function assertTelemetryEvents(extensionId, expectedEvents) {
-  const snapshot = Services.telemetry.snapshotEvents(
-    Ci.nsITelemetry.DATASET_PRERELEASE_CHANNELS,
-    true
-  );
-
-  ok(
-    snapshot.parent && snapshot.parent.length > 0,
-    "Got parent telemetry events in the snapshot"
-  );
-
-  const migrateEvents = snapshot.parent
-    .filter(([timestamp, category, method, object, value]) => {
-      return (
-        category === EVENT_CATEGORY &&
-        EVENT_METHODS.includes(method) &&
-        object === EVENT_OBJECT &&
-        value === extensionId
-      );
-    })
-    .map(event => {
-      return { method: event[2], extra: event[5] };
-    });
-
-  Assert.deepEqual(
-    migrateEvents,
-    expectedEvents,
-    "Got the expected telemetry events"
-  );
+function assertTelemetryEvents(expectedEvents) {
+  TelemetryTestUtils.assertEvents(expectedEvents, {
+    category: EVENT_CATEGORY,
+    method: EVENT_METHOD,
+    object: EVENT_OBJECT,
+  });
 }
 
-add_task(async function setup() {
+add_setup(async function setup() {
   Services.prefs.setBoolPref(ExtensionStorageIDB.BACKEND_ENABLED_PREF, true);
 
   await promiseStartupManager();
@@ -132,6 +113,106 @@ add_task(async function setup() {
   registerCleanupFunction(() => {
     Services.telemetry.canRecordBase = oldCanRecordBase;
   });
+
+  // Clear any telemetry events collected so far.
+  Services.telemetry.clearEvents();
+});
+
+// Test that for newly installed extension the IDB backend is enabled without
+// any data migration.
+add_task(async function test_no_migration_for_newly_installed_extensions() {
+  const EXTENSION_ID = "test-no-data-migration@mochi.test";
+
+  await createExtensionJSONFileWithData(EXTENSION_ID, {
+    test_old_data: "test_old_value",
+  });
+
+  const extension = ExtensionTestUtils.loadExtension({
+    useAddonManager: "temporary",
+    manifest: {
+      permissions: ["storage"],
+      browser_specific_settings: { gecko: { id: EXTENSION_ID } },
+    },
+    async background() {
+      const data = await browser.storage.local.get();
+      browser.test.assertEq(
+        Object.keys(data).length,
+        0,
+        "Expect the storage.local store to be empty"
+      );
+      browser.test.sendMessage("test-stored-data:done");
+    },
+  });
+
+  await extension.startup();
+  equal(
+    ExtensionStorageIDB.isMigratedExtension(extension),
+    true,
+    "The newly installed test extension is marked as migrated"
+  );
+  await extension.awaitMessage("test-stored-data:done");
+  await extension.unload();
+
+  // Verify that no data migration have been needed on the newly installed
+  // extension, by asserting that no telemetry events has been collected.
+  await TelemetryTestUtils.assertEvents([], TELEMETRY_EVENTS_FILTER);
+});
+
+// Test that the data migration is still running for a newly installed extension
+// if keepStorageOnUninstall is true.
+add_task(async function test_data_migration_on_keep_storage_on_uninstall() {
+  Services.prefs.setBoolPref(LEAVE_STORAGE_PREF, true);
+
+  // Store some fake data in the storage.local file backend before starting the extension.
+  const EXTENSION_ID = "new-extension-on-keep-storage-on-uninstall@mochi.test";
+  await createExtensionJSONFileWithData(EXTENSION_ID, {
+    test_key_string: "test_value",
+  });
+
+  const extension = ExtensionTestUtils.loadExtension({
+    useAddonManager: "temporary",
+    manifest: {
+      permissions: ["storage"],
+      browser_specific_settings: { gecko: { id: EXTENSION_ID } },
+    },
+    async background() {
+      const storedData = await browser.storage.local.get();
+      browser.test.assertEq(
+        "test_value",
+        storedData.test_key_string,
+        "Got the expected data after the storage.local data migration"
+      );
+      browser.test.sendMessage("storage-local-data-migrated");
+    },
+  });
+
+  await extension.startup();
+  await extension.awaitMessage("storage-local-data-migrated");
+  equal(
+    ExtensionStorageIDB.isMigratedExtension(extension),
+    true,
+    "The newly installed test extension is marked as migrated"
+  );
+  await extension.unload();
+
+  // Verify that the expected telemetry has been recorded.
+  await TelemetryTestUtils.assertEvents(
+    [
+      {
+        method: "migrateResult",
+        value: EXTENSION_ID,
+        extra: {
+          backend: "IndexedDB",
+          data_migrated: "y",
+          has_jsonfile: "y",
+          has_olddata: "y",
+        },
+      },
+    ],
+    TELEMETRY_EVENTS_FILTER
+  );
+
+  Services.prefs.clearUserPref(LEAVE_STORAGE_PREF);
 });
 
 // Test that the old data is migrated successfully to the new storage backend
@@ -186,21 +267,39 @@ add_task(async function test_storage_local_data_migration() {
     useAddonManager: "temporary",
     manifest: {
       permissions: ["storage"],
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION_ID,
         },
       },
     },
-    background,
   };
 
   let extension = ExtensionTestUtils.loadExtension(extensionDefinition);
 
+  // Install the extension while the storage.local IDB backend is disabled.
+  Services.prefs.setBoolPref(ExtensionStorageIDB.BACKEND_ENABLED_PREF, false);
   await extension.startup();
 
+  ok(
+    !ExtensionStorageIDB.isMigratedExtension(extension),
+    "The test extension should be using the JSONFile backend"
+  );
+
+  // Enabled the storage.local IDB backend and upgrade the extension.
+  Services.prefs.setBoolPref(ExtensionStorageIDB.BACKEND_ENABLED_PREF, true);
+  await extension.upgrade({
+    ...extensionDefinition,
+    background,
+  });
+
   await extension.awaitMessage("storage-local-data-migrated");
 
+  ok(
+    ExtensionStorageIDB.isMigratedExtension(extension),
+    "The test extension should be using the IndexedDB backend"
+  );
+
   const storagePrincipal = ExtensionStorageIDB.getStoragePrincipal(
     extension.extension
   );
@@ -214,13 +313,13 @@ add_task(async function test_storage_local_data_migration() {
   );
 
   equal(
-    await OS.File.exists(oldStorageFilename),
+    await IOUtils.exists(oldStorageFilename),
     false,
     "The old json storage file name should not exist anymore"
   );
 
   equal(
-    await OS.File.exists(`${oldStorageFilename}.migrated`),
+    await IOUtils.exists(`${oldStorageFilename}.migrated`),
     true,
     "The old json storage file name should have been renamed as .migrated"
   );
@@ -237,9 +336,10 @@ add_task(async function test_storage_local_data_migration() {
   assertMigrationHistogramCount("success", 1);
   assertMigrationHistogramCount("failure", 0);
 
-  assertTelemetryEvents(EXTENSION_ID, [
+  assertTelemetryEvents([
     {
       method: "migrateResult",
+      value: EXTENSION_ID,
       extra: {
         backend: "IndexedDB",
         data_migrated: "y",
@@ -249,8 +349,6 @@ add_task(async function test_storage_local_data_migration() {
     },
   ]);
 
-  await extension.unload();
-
   equal(
     Services.prefs.getBoolPref(
       `${IDB_MIGRATED_PREF_BRANCH}.${EXTENSION_ID}`,
@@ -260,11 +358,12 @@ add_task(async function test_storage_local_data_migration() {
     `${IDB_MIGRATED_PREF_BRANCH} should still be true on keepStorageOnUninstall=true`
   );
 
-  // Re-install the extension and check that no telemetry events are being sent
+  // Upgrade the extension and check that no telemetry events are being sent
   // for an already migrated extension.
-  extension = ExtensionTestUtils.loadExtension(extensionDefinition);
-
-  await extension.startup();
+  await extension.upgrade({
+    ...extensionDefinition,
+    background,
+  });
 
   await extension.awaitMessage("storage-local-data-migrated");
 
@@ -301,9 +400,7 @@ add_task(async function test_storage_local_data_migration() {
 // as expected.
 add_task(async function test_extensionId_trimmed_in_telemetry_event() {
   // Generated extensionId in email-like format, longer than 80 chars.
-  const EXTENSION_ID = `long.extension.id@${Array(80)
-    .fill("a")
-    .join("")}`;
+  const EXTENSION_ID = `long.extension.id@${Array(80).fill("a").join("")}`;
 
   const data = { test_key_string: "test_value" };
 
@@ -325,13 +422,16 @@ add_task(async function test_extensionId_trimmed_in_telemetry_event() {
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
       permissions: ["storage"],
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION_ID,
         },
       },
     },
     background,
+    // We don't want the (default) startupReason ADDON_INSTALL because
+    // that automatically sets the migrated pref and skips migration.
+    startupReason: "APP_STARTUP",
   });
 
   await extension.startup();
@@ -346,9 +446,10 @@ add_task(async function test_extensionId_trimmed_in_telemetry_event() {
     "The trimmed version of the extensionId should be 80 chars long"
   );
 
-  assertTelemetryEvents(expectedTrimmedExtensionId, [
+  assertTelemetryEvents([
     {
       method: "migrateResult",
+      value: expectedTrimmedExtensionId,
       extra: {
         backend: "IndexedDB",
         data_migrated: "y",
@@ -365,7 +466,7 @@ add_task(async function test_extensionId_trimmed_in_telemetry_event() {
 // can't be successfully migrated to the new storage backend, then:
 // - the new storage backend for that extension is still initialized and enabled
 // - any new data is being stored in the new backend
-// - the old file is being renamed (with the `.corrupted` suffix that JSONFile.jsm
+// - the old file is being renamed (with the `.corrupted` suffix that JSONFile.sys.mjs
 //   adds when it fails to load the data file) and still available on disk.
 add_task(async function test_storage_local_corrupted_data_migration() {
   const EXTENSION_ID = "extension-corrupted-data-migration@mozilla.org";
@@ -373,16 +474,14 @@ add_task(async function test_storage_local_corrupted_data_migration() {
   const invalidData = `{"test_key_string": "test_value1"`;
   const oldStorageFilename = ExtensionStorage.getStorageFile(EXTENSION_ID);
 
-  const profileDir = OS.Constants.Path.profileDir;
-  await OS.File.makeDir(
-    OS.Path.join(profileDir, "browser-extension-data", EXTENSION_ID),
-    { from: profileDir, ignoreExisting: true }
+  await IOUtils.makeDirectory(
+    PathUtils.join(PathUtils.profileDir, "browser-extension-data", EXTENSION_ID)
   );
 
   // Write the json file with some invalid data.
-  await OS.File.writeAtomic(oldStorageFilename, invalidData, { flush: true });
+  await IOUtils.writeUTF8(oldStorageFilename, invalidData, { flush: true });
   equal(
-    await OS.File.read(oldStorageFilename, { encoding: "utf-8" }),
+    await IOUtils.readUTF8(oldStorageFilename),
     invalidData,
     "The old json file has been overwritten with invalid data"
   );
@@ -408,13 +507,16 @@ add_task(async function test_storage_local_corrupted_data_migration() {
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
       permissions: ["storage"],
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION_ID,
         },
       },
     },
     background,
+    // We don't want the (default) startupReason ADDON_INSTALL because
+    // that automatically sets the migrated pref and skips migration.
+    startupReason: "APP_STARTUP",
   });
 
   await extension.startup();
@@ -434,7 +536,7 @@ add_task(async function test_storage_local_corrupted_data_migration() {
   );
 
   equal(
-    await OS.File.exists(`${oldStorageFilename}.corrupt`),
+    await IOUtils.exists(`${oldStorageFilename}.corrupt`),
     true,
     "The old json storage should still be available if failed to be read"
   );
@@ -454,9 +556,10 @@ add_task(async function test_storage_local_corrupted_data_migration() {
   assertMigrationHistogramCount("success", 1);
   assertMigrationHistogramCount("failure", 0);
 
-  assertTelemetryEvents(EXTENSION_ID, [
+  assertTelemetryEvents([
     {
       method: "migrateResult",
+      value: EXTENSION_ID,
       extra: {
         backend: "IndexedDB",
         data_migrated: "y",
@@ -475,16 +578,14 @@ add_task(async function test_storage_local_data_migration_failure() {
   const EXTENSION_ID = "extension-data-migration-failure@mozilla.org";
 
   // Create the file under the expected directory tree.
-  const {
-    jsonFile,
-    oldStorageFilename,
-  } = await createExtensionJSONFileWithData(EXTENSION_ID, {});
+  const { jsonFile, oldStorageFilename } =
+    await createExtensionJSONFileWithData(EXTENSION_ID, {});
 
   // Store a fake invalid value which is going to fail to be saved into IndexedDB
   // (because it can't be cloned and it is going to raise a DataCloneError), which
   // will trigger a data migration failure that we expect to increment the related
   // telemetry histogram.
-  jsonFile.data.set("fake_invalid_key", new Error());
+  jsonFile.data.set("fake_invalid_key", function () {});
 
   async function background() {
     await browser.storage.local.set({
@@ -498,13 +599,16 @@ add_task(async function test_storage_local_data_migration_failure() {
   let extension = ExtensionTestUtils.loadExtension({
     manifest: {
       permissions: ["storage"],
-      applications: {
+      browser_specific_settings: {
         gecko: {
           id: EXTENSION_ID,
         },
       },
     },
     background,
+    // We don't want the (default) startupReason ADDON_INSTALL because
+    // that automatically sets the migrated pref and skips migration.
+    startupReason: "APP_STARTUP",
   });
 
   await extension.startup();
@@ -522,16 +626,17 @@ add_task(async function test_storage_local_data_migration_failure() {
     "No data stored in the ExtensionStorageIDB backend as expected"
   );
   equal(
-    await OS.File.exists(oldStorageFilename),
+    await IOUtils.exists(oldStorageFilename),
     true,
     "The old json storage should still be available if failed to be read"
   );
 
   await extension.unload();
 
-  assertTelemetryEvents(EXTENSION_ID, [
+  assertTelemetryEvents([
     {
       method: "migrateResult",
+      value: EXTENSION_ID,
       extra: {
         backend: "JSONFile",
         data_migrated: "n",
@@ -546,6 +651,56 @@ add_task(async function test_storage_local_data_migration_failure() {
   assertMigrationHistogramCount("failure", 1);
 });
 
+add_task(async function test_migration_aborted_on_shutdown() {
+  const EXTENSION_ID = "test-migration-aborted-on-shutdown@mochi.test";
+  let extension = ExtensionTestUtils.loadExtension({
+    manifest: {
+      permissions: ["storage"],
+      browser_specific_settings: {
+        gecko: {
+          id: EXTENSION_ID,
+        },
+      },
+    },
+    // We don't want the (default) startupReason ADDON_INSTALL because
+    // that automatically sets the migrated pref and skips migration.
+    startupReason: "APP_STARTUP",
+  });
+
+  await extension.startup();
+
+  equal(
+    extension.extension.hasShutdown,
+    false,
+    "The extension is still running"
+  );
+
+  await extension.unload();
+  equal(extension.extension.hasShutdown, true, "The extension has shutdown");
+
+  // Trigger a data migration after the extension has been unloaded.
+  const result = await ExtensionStorageIDB.selectBackend({
+    extension: extension.extension,
+  });
+  Assert.deepEqual(
+    result,
+    { backendEnabled: false },
+    "Expect migration to have been aborted"
+  );
+  TelemetryTestUtils.assertEvents(
+    [
+      {
+        value: EXTENSION_ID,
+        extra: {
+          backend: "JSONFile",
+          error_name: "DataMigrationAbortedError",
+        },
+      },
+    ],
+    TELEMETRY_EVENTS_FILTER
+  );
+});
+
 add_task(async function test_storage_local_data_migration_clear_pref() {
   Services.prefs.clearUserPref(LEAVE_STORAGE_PREF);
   Services.prefs.clearUserPref(LEAVE_UUID_PREF);
@@ -553,3 +708,83 @@ add_task(async function test_storage_local_data_migration_clear_pref() {
   await promiseShutdownManager();
   await TelemetryController.testShutdown();
 });
+
+add_task(async function setup_quota_manager_testing_prefs() {
+  Services.prefs.setBoolPref("dom.quotaManager.testing", true);
+  Services.prefs.setIntPref(
+    "dom.quotaManager.temporaryStorage.fixedLimit",
+    100
+  );
+  await promiseQuotaManagerServiceReset();
+});
+
+add_task(
+  // TODO: temporarily disabled because it currently perma-fails on
+  // android builds (Bug 1564871)
+  { skip_if: () => AppConstants.platform === "android" },
+  // eslint-disable-next-line no-use-before-define
+  test_quota_exceeded_while_migrating_data
+);
+async function test_quota_exceeded_while_migrating_data() {
+  const EXT_ID = "test-data-migration-stuck@mochi.test";
+  const dataSize = 1000 * 1024;
+
+  await createExtensionJSONFileWithData(EXT_ID, {
+    data: new Array(dataSize).fill("x").join(""),
+  });
+
+  const extension = ExtensionTestUtils.loadExtension({
+    manifest: {
+      permissions: ["storage"],
+      browser_specific_settings: { gecko: { id: EXT_ID } },
+    },
+    background() {
+      browser.test.onMessage.addListener(async (msg, dataSize) => {
+        if (msg !== "verify-stored-data") {
+          return;
+        }
+        const res = await browser.storage.local.get();
+        browser.test.assertEq(
+          res.data && res.data.length,
+          dataSize,
+          "Got the expected data"
+        );
+        browser.test.sendMessage("verify-stored-data:done");
+      });
+
+      browser.test.sendMessage("bg-page:ready");
+    },
+    // We don't want the (default) startupReason ADDON_INSTALL because
+    // that automatically sets the migrated pref and skips migration.
+    startupReason: "APP_STARTUP",
+  });
+
+  await extension.startup();
+  await extension.awaitMessage("bg-page:ready");
+
+  extension.sendMessage("verify-stored-data", dataSize);
+  await extension.awaitMessage("verify-stored-data:done");
+
+  await ok(
+    !ExtensionStorageIDB.isMigratedExtension(extension),
+    "The extension falls back to the JSONFile backend because of the migration failure"
+  );
+  await extension.unload();
+
+  TelemetryTestUtils.assertEvents(
+    [
+      {
+        value: EXT_ID,
+        extra: {
+          backend: "JSONFile",
+          error_name: "QuotaExceededError",
+        },
+      },
+    ],
+    TELEMETRY_EVENTS_FILTER
+  );
+
+  Services.prefs.clearUserPref("dom.quotaManager.temporaryStorage.fixedLimit");
+  await promiseQuotaManagerServiceClear();
+  Services.prefs.clearUserPref("dom.quotaManager.testing");
+}
