@@ -6,19 +6,22 @@
 
 #include "StreamFilterParent.h"
 
-#include "mozilla/ScopeExit.h"
+#include "HttpChannelChild.h"
+#include "mozilla/ExtensionPolicyService.h"
 #include "mozilla/Unused.h"
 #include "mozilla/dom/ContentParent.h"
 #include "mozilla/net/ChannelEventQueue.h"
+#include "mozilla/StaticPrefs_extensions.h"
 #include "nsHttpChannel.h"
 #include "nsIChannel.h"
-#include "nsIHttpChannelInternal.h"
 #include "nsIInputStream.h"
 #include "nsITraceableChannel.h"
 #include "nsProxyRelease.h"
 #include "nsQueryObject.h"
 #include "nsSocketTransportService2.h"
 #include "nsStringStream.h"
+#include "mozilla/net/DocumentChannelChild.h"
+#include "nsIViewSourceChannel.h"
 
 namespace mozilla {
 namespace extensions {
@@ -90,7 +93,7 @@ class ChannelEventRunnable final : public ChannelEventWrapper {
  *****************************************************************************/
 
 StreamFilterParent::StreamFilterParent()
-    : mMainThread(GetCurrentThreadEventTarget()),
+    : mMainThread(GetCurrentSerialEventTarget()),
       mIOThread(mMainThread),
       mQueue(new ChannelEventQueue(static_cast<nsIStreamListener*>(this))),
       mBufferMutex("StreamFilter buffer mutex"),
@@ -101,19 +104,17 @@ StreamFilterParent::StreamFilterParent()
       mState(State::Uninitialized) {}
 
 StreamFilterParent::~StreamFilterParent() {
-  NS_ReleaseOnMainThreadSystemGroup("StreamFilterParent::mChannel",
-                                    mChannel.forget());
-  NS_ReleaseOnMainThreadSystemGroup("StreamFilterParent::mLoadGroup",
-                                    mLoadGroup.forget());
-  NS_ReleaseOnMainThreadSystemGroup("StreamFilterParent::mOrigListener",
-                                    mOrigListener.forget());
-  NS_ReleaseOnMainThreadSystemGroup("StreamFilterParent::mContext",
-                                    mContext.forget());
+  NS_ReleaseOnMainThread("StreamFilterParent::mChannel", mChannel.forget());
+  NS_ReleaseOnMainThread("StreamFilterParent::mLoadGroup", mLoadGroup.forget());
+  NS_ReleaseOnMainThread("StreamFilterParent::mOrigListener",
+                         mOrigListener.forget());
+  NS_ReleaseOnMainThread("StreamFilterParent::mContext", mContext.forget());
+  mQueue->NotifyReleasingOwner();
 }
 
-bool StreamFilterParent::Create(dom::ContentParent* aContentParent,
-                                uint64_t aChannelId, const nsAString& aAddonId,
-                                Endpoint<PStreamFilterChild>* aEndpoint) {
+auto StreamFilterParent::Create(dom::ContentParent* aContentParent,
+                                uint64_t aChannelId, const nsAString& aAddonId)
+    -> RefPtr<ChildEndpointPromise> {
   AssertIsMainThread();
 
   auto& webreq = WebRequestService::GetSingleton();
@@ -123,25 +124,28 @@ bool StreamFilterParent::Create(dom::ContentParent* aContentParent,
       webreq.GetTraceableChannel(aChannelId, addonId, aContentParent);
 
   RefPtr<mozilla::net::nsHttpChannel> chan = do_QueryObject(channel);
-  NS_ENSURE_TRUE(chan, false);
-
-  auto channelPid = chan->ProcessId();
-  NS_ENSURE_TRUE(channelPid, false);
-
-  Endpoint<PStreamFilterParent> parent;
-  Endpoint<PStreamFilterChild> child;
-  nsresult rv = PStreamFilter::CreateEndpoints(
-      channelPid,
-      aContentParent ? aContentParent->OtherPid() : base::GetCurrentProcId(),
-      &parent, &child);
-  NS_ENSURE_SUCCESS(rv, false);
-
-  if (!chan->AttachStreamFilter(std::move(parent))) {
-    return false;
+  if (!chan) {
+    return ChildEndpointPromise::CreateAndReject(false, __func__);
   }
 
-  *aEndpoint = std::move(child);
-  return true;
+  nsCOMPtr<nsIChannel> genChan(do_QueryInterface(channel));
+  if (!StaticPrefs::extensions_filterResponseServiceWorkerScript_disabled() &&
+      ChannelWrapper::IsServiceWorkerScript(genChan)) {
+    RefPtr<extensions::WebExtensionPolicy> addonPolicy =
+        ExtensionPolicyService::GetSingleton().GetByID(aAddonId);
+
+    if (!addonPolicy ||
+        !addonPolicy->HasPermission(
+            nsGkAtoms::webRequestFilterResponse_serviceWorkerScript)) {
+      return ChildEndpointPromise::CreateAndReject(false, __func__);
+    }
+  }
+
+  // Disable alt-data for extension stream listeners.
+  nsCOMPtr<nsIHttpChannelInternal> internal(do_QueryObject(channel));
+  internal->DisableAltDataCache();
+
+  return chan->AttachStreamFilter();
 }
 
 /* static */
@@ -155,10 +159,33 @@ void StreamFilterParent::Attach(nsIChannel* aChannel,
                                           std::move(aEndpoint)),
       NS_DISPATCH_NORMAL);
 
-  self->Init(aChannel);
+  // If aChannel is a HttpChannelChild, ask HttpChannelChild to hold a weak
+  // reference on this StreamFilterParent. Such that HttpChannelChild has a
+  // chance to disconnect this StreamFilterParent if internal redirection
+  // happens, i.e. ServiceWorker fallback redirection.
+  RefPtr<net::HttpChannelChild> channelChild = do_QueryObject(aChannel);
+  if (channelChild) {
+    channelChild->RegisterStreamFilter(self);
+  }
 
-  // IPC owns this reference now.
-  Unused << self.forget();
+  self->Init(aChannel);
+}
+
+void StreamFilterParent::Disconnect(const nsACString& aReason) {
+  AssertIsMainThread();
+  MOZ_DIAGNOSTIC_ASSERT(mBeforeOnStartRequest);
+
+  mDisconnected = true;
+
+  nsAutoCString reason(aReason);
+
+  RefPtr<StreamFilterParent> self(this);
+  RunOnActorThread(FUNC, [self, reason] {
+    if (self->IPCActive()) {
+      self->mState = State::Disconnected;
+      self->CheckResult(self->SendError(reason));
+    }
+  });
 }
 
 void StreamFilterParent::Bind(ParentEndpoint&& aEndpoint) {
@@ -169,9 +196,23 @@ void StreamFilterParent::Init(nsIChannel* aChannel) {
   mChannel = aChannel;
 
   nsCOMPtr<nsITraceableChannel> traceable = do_QueryInterface(aChannel);
-  MOZ_RELEASE_ASSERT(traceable);
+  if (MOZ_UNLIKELY(!traceable)) {
+    // nsViewSourceChannel is not nsITraceableChannel, but wraps one. Unwrap it.
+    nsCOMPtr<nsIViewSourceChannel> vsc = do_QueryInterface(aChannel);
+    if (vsc) {
+      traceable = do_QueryObject(vsc->GetInnerChannel());
+      // OnStartRequest etc. is passed the unwrapped channel, so update mChannel
+      // to prevent OnStartRequest from mistaking it for a redirect, which would
+      // close the filter.
+      mChannel = do_QueryObject(traceable);
+    }
+    // TODO bug 1683403: Replace assertion; Close StreamFilter instead.
+    MOZ_RELEASE_ASSERT(traceable);
+  }
 
-  nsresult rv = traceable->SetNewListener(this, getter_AddRefs(mOrigListener));
+  nsresult rv =
+      traceable->SetNewListener(this, /* aMustApplyContentConversion = */ true,
+                                getter_AddRefs(mOrigListener));
   MOZ_RELEASE_ASSERT(NS_SUCCEEDED(rv));
 }
 
@@ -201,14 +242,17 @@ void StreamFilterParent::Broken() {
   switch (mState) {
     case State::Initialized:
     case State::TransferringData:
-    case State::Suspended:
+    case State::Suspended: {
       mState = State::Disconnecting;
-      if (mChannel) {
-        mChannel->Cancel(NS_ERROR_FAILURE);
-      }
+      RefPtr<StreamFilterParent> self(this);
+      RunOnMainThread(FUNC, [=] {
+        if (self->mChannel) {
+          self->mChannel->Cancel(NS_ERROR_FAILURE);
+        }
+      });
 
       FinishDisconnect();
-      break;
+    } break;
 
     default:
       break;
@@ -323,15 +367,18 @@ void StreamFilterParent::FinishDisconnect() {
     self->FlushBufferedData();
 
     RunOnMainThread(FUNC, [=] {
-      if (self->mLoadGroup) {
+      if (self->mReceivedStop && !self->mSentStop) {
+        nsresult rv = self->EmitStopRequest(NS_OK);
+        Unused << NS_WARN_IF(NS_FAILED(rv));
+      } else if (self->mLoadGroup && !self->mDisconnected) {
         Unused << self->mLoadGroup->RemoveRequest(self, nullptr, NS_OK);
       }
+      self->mDisconnected = true;
     });
 
     RunOnActorThread(FUNC, [=] {
       if (self->mState != State::Closed) {
         self->mState = State::Disconnected;
-        self->mDisconnected = true;
       }
     });
   });
@@ -361,7 +408,7 @@ nsresult StreamFilterParent::Write(Data& aData) {
   nsCOMPtr<nsIInputStream> stream;
   nsresult rv = NS_NewByteInputStream(
       getter_AddRefs(stream),
-      MakeSpan(reinterpret_cast<char*>(aData.Elements()), aData.Length()),
+      Span(reinterpret_cast<char*>(aData.Elements()), aData.Length()),
       NS_ASSIGNMENT_DEPEND);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -379,12 +426,14 @@ nsresult StreamFilterParent::Write(Data& aData) {
 
 NS_IMETHODIMP
 StreamFilterParent::GetName(nsACString& aName) {
+  AssertIsMainThread();
   MOZ_ASSERT(mChannel);
   return mChannel->GetName(aName);
 }
 
 NS_IMETHODIMP
 StreamFilterParent::GetStatus(nsresult* aStatus) {
+  AssertIsMainThread();
   MOZ_ASSERT(mChannel);
   return mChannel->GetStatus(aStatus);
 }
@@ -403,20 +452,36 @@ StreamFilterParent::IsPending(bool* aIsPending) {
   return NS_OK;
 }
 
+NS_IMETHODIMP StreamFilterParent::SetCanceledReason(const nsACString& aReason) {
+  return SetCanceledReasonImpl(aReason);
+}
+
+NS_IMETHODIMP StreamFilterParent::GetCanceledReason(nsACString& aReason) {
+  return GetCanceledReasonImpl(aReason);
+}
+
+NS_IMETHODIMP StreamFilterParent::CancelWithReason(nsresult aStatus,
+                                                   const nsACString& aReason) {
+  return CancelWithReasonImpl(aStatus, aReason);
+}
+
 NS_IMETHODIMP
 StreamFilterParent::Cancel(nsresult aResult) {
+  AssertIsMainThread();
   MOZ_ASSERT(mChannel);
   return mChannel->Cancel(aResult);
 }
 
 NS_IMETHODIMP
 StreamFilterParent::Suspend() {
+  AssertIsMainThread();
   MOZ_ASSERT(mChannel);
   return mChannel->Suspend();
 }
 
 NS_IMETHODIMP
 StreamFilterParent::Resume() {
+  AssertIsMainThread();
   MOZ_ASSERT(mChannel);
   return mChannel->Resume();
 }
@@ -434,6 +499,7 @@ StreamFilterParent::SetLoadGroup(nsILoadGroup* aLoadGroup) {
 
 NS_IMETHODIMP
 StreamFilterParent::GetLoadFlags(nsLoadFlags* aLoadFlags) {
+  AssertIsMainThread();
   MOZ_ASSERT(mChannel);
   MOZ_TRY(mChannel->GetLoadFlags(aLoadFlags));
   *aLoadFlags &= ~nsIChannel::LOAD_DOCUMENT_URI;
@@ -442,10 +508,21 @@ StreamFilterParent::GetLoadFlags(nsLoadFlags* aLoadFlags) {
 
 NS_IMETHODIMP
 StreamFilterParent::SetLoadFlags(nsLoadFlags aLoadFlags) {
+  AssertIsMainThread();
   MOZ_ASSERT(mChannel);
   return mChannel->SetLoadFlags(aLoadFlags);
 }
 
+NS_IMETHODIMP
+StreamFilterParent::GetTRRMode(nsIRequest::TRRMode* aTRRMode) {
+  return GetTRRModeImpl(aTRRMode);
+}
+
+NS_IMETHODIMP
+StreamFilterParent::SetTRRMode(nsIRequest::TRRMode aTRRMode) {
+  return SetTRRModeImpl(aTRRMode);
+}
+
 /*****************************************************************************
  * nsIStreamListener
  *****************************************************************************/
@@ -454,16 +531,50 @@ NS_IMETHODIMP
 StreamFilterParent::OnStartRequest(nsIRequest* aRequest) {
   AssertIsMainThread();
 
+  // Always reset mChannel if aRequest is different.  Various calls in
+  // StreamFilterParent will use mChannel, but aRequest is *always* the
+  // right channel to use at this point.
+  //
+  // For ALL redirections, we will disconnect this listener.  Extensions
+  // will create a new filter if they need it.
+  mBeforeOnStartRequest = false;
   if (aRequest != mChannel) {
-    mDisconnected = true;
+    nsCOMPtr<nsIChannel> channel = do_QueryInterface(aRequest);
+    nsCOMPtr<nsILoadInfo> loadInfo = channel ? channel->LoadInfo() : nullptr;
+    mChannel = channel;
 
-    RefPtr<StreamFilterParent> self(this);
-    RunOnActorThread(FUNC, [=] {
-      if (self->IPCActive()) {
-        self->mState = State::Disconnected;
-        CheckResult(self->SendError(NS_LITERAL_CSTRING("Channel redirected")));
-      }
-    });
+    if (!(loadInfo &&
+          loadInfo->RedirectChainIncludingInternalRedirects().IsEmpty())) {
+      mDisconnected = true;
+      mDisconnectedByOnStartRequest = true;
+
+      RefPtr<StreamFilterParent> self(this);
+      RunOnActorThread(FUNC, [=] {
+        if (self->IPCActive()) {
+          self->mState = State::Disconnected;
+          CheckResult(self->SendError("Channel redirected"_ns));
+        }
+      });
+    }
+  }
+
+  // Check if alterate cached data is being sent, if so we receive un-decoded
+  // data and we must disconnect the filter and send an error to the extension.
+  if (!mDisconnected) {
+    RefPtr<net::HttpBaseChannel> chan = do_QueryObject(aRequest);
+    if (chan && chan->IsDeliveringAltData()) {
+      mDisconnected = true;
+      mDisconnectedByOnStartRequest = true;
+
+      RefPtr<StreamFilterParent> self(this);
+      RunOnActorThread(FUNC, [=] {
+        if (self->IPCActive()) {
+          self->mState = State::Disconnected;
+          CheckResult(
+              self->SendError("Channel is delivering cached alt-data"_ns));
+        }
+      });
+    }
   }
 
   if (!mDisconnected) {
@@ -479,7 +590,7 @@ StreamFilterParent::OnStartRequest(nsIRequest* aRequest) {
   // that we get the final delivery target after any retargeting that it may do.
   if (nsCOMPtr<nsIThreadRetargetableRequest> req =
           do_QueryInterface(aRequest)) {
-    nsCOMPtr<nsIEventTarget> thread;
+    nsCOMPtr<nsISerialEventTarget> thread;
     Unused << req->GetDeliveryTarget(getter_AddRefs(thread));
     if (thread) {
       mIOThread = std::move(thread);
@@ -506,6 +617,7 @@ StreamFilterParent::OnStartRequest(nsIRequest* aRequest) {
 NS_IMETHODIMP
 StreamFilterParent::OnStopRequest(nsIRequest* aRequest, nsresult aStatusCode) {
   AssertIsMainThread();
+  MOZ_ASSERT(aRequest == mChannel);
 
   mReceivedStop = true;
   if (mDisconnected) {
@@ -516,6 +628,16 @@ StreamFilterParent::OnStopRequest(nsIRequest* aRequest, nsresult aStatusCode) {
   RunOnActorThread(FUNC, [=] {
     if (self->IPCActive()) {
       self->CheckResult(self->SendStopRequest(aStatusCode));
+    } else if (self->mState != State::Disconnecting) {
+      // If we're currently disconnecting, then we'll emit a stop
+      // request at the end of that process. Otherwise we need to
+      // manually emit one here, since we won't be getting a response
+      // from the child.
+      RunOnMainThread(FUNC, [=] {
+        if (!self->mSentStop) {
+          self->EmitStopRequest(aStatusCode);
+        }
+      });
     }
   });
   return NS_OK;
@@ -553,7 +675,7 @@ StreamFilterParent::OnDataAvailable(nsIRequest* aRequest,
                                     uint64_t aOffset, uint32_t aCount) {
   AssertIsIOThread();
 
-  if (mDisconnected) {
+  if (mDisconnectedByOnStartRequest || mState == State::Disconnected) {
     // If we're offloading data in a thread pool, it's possible that we'll
     // have buffered some additional data while waiting for the buffer to
     // flush. So, if there's any buffered data left, flush that before we
@@ -609,16 +731,6 @@ nsresult StreamFilterParent::FlushBufferedData() {
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  if (mReceivedStop && !mSentStop) {
-    RefPtr<StreamFilterParent> self(this);
-    RunOnMainThread(FUNC, [=] {
-      if (!mSentStop) {
-        nsresult rv = self->EmitStopRequest(NS_OK);
-        Unused << NS_WARN_IF(NS_FAILED(rv));
-      }
-    });
-  }
-
   return NS_OK;
 }
 
@@ -636,7 +748,7 @@ bool StreamFilterParent::IsActorThread() {
 
 void StreamFilterParent::AssertIsActorThread() { MOZ_ASSERT(IsActorThread()); }
 
-nsIEventTarget* StreamFilterParent::IOThread() { return mIOThread; }
+nsISerialEventTarget* StreamFilterParent::IOThread() { return mIOThread; }
 
 bool StreamFilterParent::IsIOThread() { return mIOThread->IsOnCurrentThread(); }
 
@@ -699,10 +811,6 @@ void StreamFilterParent::ActorDestroy(ActorDestroyReason aWhy) {
   }
 }
 
-void StreamFilterParent::DeallocPStreamFilterParent() {
-  RefPtr<StreamFilterParent> self = dont_AddRef(this);
-}
-
 NS_INTERFACE_MAP_BEGIN(StreamFilterParent)
   NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
   NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
