@@ -3,10 +3,9 @@
 
 "use strict";
 
-const { KeyValueService } = ChromeUtils.import(
-  "resource://gre/modules/kvstore.jsm"
+const { KeyValueService } = ChromeUtils.importESModule(
+  "resource://gre/modules/kvstore.sys.mjs"
 );
-const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
 
 function run_test() {
   do_get_profile();
@@ -14,8 +13,8 @@ function run_test() {
 }
 
 async function makeDatabaseDir(name) {
-  const databaseDir = OS.Path.join(OS.Constants.Path.profileDir, name);
-  await OS.File.makeDir(databaseDir, { from: OS.Constants.Path.profileDir });
+  const databaseDir = PathUtils.join(PathUtils.profileDir, name);
+  await IOUtils.makeDirectory(databaseDir);
   return databaseDir;
 }
 
@@ -33,22 +32,16 @@ add_task(async function getOrCreate() {
   Assert.ok(database);
 
   // Test creating a database with a nonexistent path.
-  const nonexistentDir = OS.Path.join(
-    OS.Constants.Path.profileDir,
-    "nonexistent"
-  );
+  const nonexistentDir = PathUtils.join(PathUtils.profileDir, "nonexistent");
   await Assert.rejects(
     KeyValueService.getOrCreate(nonexistentDir, "db"),
-    /DirectoryDoesNotExistError/
+    /UnsuitableEnvironmentPath/
   );
 
   // Test creating a database with a non-normalized but fully-qualified path.
   let nonNormalizedDir = await makeDatabaseDir("non-normalized");
-  nonNormalizedDir = OS.Path.join(
-    nonNormalizedDir,
-    "..",
-    ".",
-    "non-normalized"
+  nonNormalizedDir = [nonNormalizedDir, "..", ".", "non-normalized"].join(
+    Services.appinfo.OS === "WINNT" ? "\\" : "/"
   );
   Assert.ok(await KeyValueService.getOrCreate(nonNormalizedDir, "db"));
 });
@@ -117,6 +110,24 @@ add_task(async function putGetHasDelete() {
   Assert.strictEqual(await database.get("bool-key"), null);
 });
 
+add_task(async function putWithResizing() {
+  const databaseDir = await makeDatabaseDir("putWithResizing");
+  const database = await KeyValueService.getOrCreate(databaseDir, "db");
+
+  // The default store size is 1MB, putting key/value pairs bigger than that
+  // would trigger auto resizing.
+  const base = "A humongous string in 32 bytes!!";
+  const val1M = base.repeat(32768);
+  const val2M = val1M.repeat(2);
+  Assert.strictEqual(await database.put("A-1M-value", val1M), undefined);
+  Assert.strictEqual(await database.put("A-2M-value", val2M), undefined);
+  Assert.strictEqual(await database.put("A-32B-value", base), undefined);
+
+  Assert.strictEqual(await database.get("A-1M-value"), val1M);
+  Assert.strictEqual(await database.get("A-2M-value"), val2M);
+  Assert.strictEqual(await database.get("A-32B-value"), base);
+});
+
 add_task(async function largeNumbers() {
   const databaseDir = await makeDatabaseDir("largeNumbers");
   const database = await KeyValueService.getOrCreate(databaseDir, "db");
@@ -238,6 +249,51 @@ add_task(async function writeManyPutOnly() {
   await test_helper(mapPairs);
 });
 
+add_task(async function writeManyLargePairsWithResizing() {
+  const databaseDir = await makeDatabaseDir("writeManyWithResizing");
+  const database = await KeyValueService.getOrCreate(databaseDir, "db");
+
+  // The default store size is 1MB, putting key/value pairs bigger than that
+  // would trigger auto resizing.
+  const base = "A humongous string in 32 bytes!!";
+  const val1M = base.repeat(32768);
+  const val2M = val1M.repeat(2);
+
+  // writeMany with an object
+  const pairs = {
+    "A-1M-value": val1M,
+    "A-32B-value": base,
+    "A-2M-value": val2M,
+  };
+
+  Assert.strictEqual(await database.writeMany(pairs), undefined);
+
+  Assert.strictEqual(await database.get("A-1M-value"), val1M);
+  Assert.strictEqual(await database.get("A-2M-value"), val2M);
+  Assert.strictEqual(await database.get("A-32B-value"), base);
+});
+
+add_task(async function writeManySmallPairsWithResizing() {
+  const databaseDir = await makeDatabaseDir("writeManyWithResizing");
+  const database = await KeyValueService.getOrCreate(databaseDir, "db");
+
+  // The default store size is 1MB, putting key/value pairs bigger than that
+  // would trigger auto resizing.
+  const base = "A humongous string in 32 bytes!!";
+  const val1K = base.repeat(32);
+  // writeMany with a key/value generator
+  function* pairMaker() {
+    for (let i = 0; i < 1024; i++) {
+      yield [`key-${i}`, val1K];
+    }
+  }
+
+  Assert.strictEqual(await database.writeMany(pairMaker()), undefined);
+  for (let i = 0; i < 1024; i++) {
+    Assert.ok(await database.has(`key-${i}`));
+  }
+});
+
 add_task(async function writeManyDeleteOnly() {
   const databaseDir = await makeDatabaseDir("writeManyDeletesOnly");
   const database = await KeyValueService.getOrCreate(databaseDir, "db");
@@ -304,7 +360,11 @@ add_task(async function writeManyPutDelete() {
     ["key5", "val5"],
   ]);
 
-  await database.writeMany([["key2", "val2"], ["key4", null], ["key5", null]]);
+  await database.writeMany([
+    ["key2", "val2"],
+    ["key4", null],
+    ["key5", null],
+  ]);
 
   Assert.strictEqual(await database.get("key1"), "val1");
   Assert.strictEqual(await database.get("key2"), "val2");
@@ -482,7 +542,10 @@ add_task(async function enumeration() {
 
   // Test enumeration to a key that does exist, which should enumerate pairs
   // whose key is less than that key.
-  await test(null, "int-key", [["bool-key", true], ["double-key", 56.78]]);
+  await test(null, "int-key", [
+    ["bool-key", true],
+    ["double-key", 56.78],
+  ]);
 
   // Test enumeration to a key that doesn't exist and is lexicographically
   // less than the least key in the database, which should enumerate
