@@ -23,14 +23,17 @@ const MAX_DATE_MS = 8640000000000000;
  *        This argument is passed to the "Assert.throws" test to determine which
  *        error is expected from the modification functions.
  */
-function checkLoginInvalid(aLoginInfo, aExpectedError) {
+async function checkLoginInvalid(aLoginInfo, aExpectedError) {
   // Try to add the new login, and verify that no data is stored.
-  Assert.throws(() => Services.logins.addLogin(aLoginInfo), aExpectedError);
+  await Assert.rejects(
+    Services.logins.addLoginAsync(aLoginInfo),
+    aExpectedError
+  );
   LoginTestUtils.checkLogins([]);
 
   // Add a login for the modification tests.
   let testLogin = TestData.formLogin({ origin: "http://modify.example.com" });
-  Services.logins.addLogin(testLogin);
+  await Services.logins.addLoginAsync(testLogin);
 
   // Try to modify the existing login using nsILoginInfo and nsIPropertyBag.
   Assert.throws(
@@ -87,16 +90,17 @@ function compareAttributes(objectA, objectB, attributes) {
 /**
  * Tests that adding logins to the database works.
  */
-add_task(function test_addLogin_removeLogin() {
+add_task(async function test_addLogin_removeLogin() {
   // Each login from the test data should be valid and added to the list.
-  for (let loginInfo of TestData.loginList()) {
-    Services.logins.addLogin(loginInfo);
-  }
+  await Services.logins.addLogins(TestData.loginList());
   LoginTestUtils.checkLogins(TestData.loginList());
 
   // Trying to add each login again should result in an error.
   for (let loginInfo of TestData.loginList()) {
-    Assert.throws(() => Services.logins.addLogin(loginInfo), /already exists/);
+    await Assert.rejects(
+      Services.logins.addLoginAsync(loginInfo),
+      /This login already exists./
+    );
   }
 
   // Removing each login should succeed.
@@ -107,6 +111,73 @@ add_task(function test_addLogin_removeLogin() {
   LoginTestUtils.checkLogins([]);
 });
 
+add_task(async function add_login_works_with_empty_array() {
+  const result = await Services.logins.addLogins([]);
+  Assert.equal(result.length, 0, "no logins added");
+});
+
+add_task(async function duplicated_logins_are_not_added() {
+  const login = TestData.formLogin({
+    username: "user",
+  });
+  await Services.logins.addLogins([login]);
+  const result = await Services.logins.addLogins([login]);
+  Assert.equal(result, 0, "no logins added");
+  Services.logins.removeAllUserFacingLogins();
+});
+
+add_task(async function logins_containing_nul_in_username_are_not_added() {
+  const result = await Services.logins.addLogins([
+    TestData.formLogin({ username: "user\0name" }),
+  ]);
+  Assert.equal(result, 0, "no logins added");
+});
+
+add_task(async function logins_containing_nul_in_password_are_not_added() {
+  const result = await Services.logins.addLogins([
+    TestData.formLogin({ password: "pass\0word" }),
+  ]);
+  Assert.equal(result, 0, "no logins added");
+});
+
+add_task(
+  async function return_value_includes_plaintext_username_and_password() {
+    const login = TestData.formLogin({});
+    const [result] = await Services.logins.addLogins([login]);
+    Assert.equal(result.username, login.username, "plaintext username is set");
+    Assert.equal(result.password, login.password, "plaintext password is set");
+    Services.logins.removeAllUserFacingLogins();
+  }
+);
+
+add_task(async function event_data_includes_plaintext_username_and_password() {
+  const login = TestData.formLogin({});
+  const TestObserver = {
+    QueryInterface: ChromeUtils.generateQI([
+      "nsIObserver",
+      "nsISupportsWeakReference",
+    ]),
+    observe(subject, topic, data) {
+      Assert.ok(subject instanceof Ci.nsILoginInfo);
+      Assert.ok(subject instanceof Ci.nsILoginMetaInfo);
+      Assert.equal(
+        subject.username,
+        login.username,
+        "plaintext username is set"
+      );
+      Assert.equal(
+        subject.password,
+        login.password,
+        "plaintext password is set"
+      );
+    },
+  };
+  Services.obs.addObserver(TestObserver, "passwordmgr-storage-changed");
+  await Services.logins.addLogins([login]);
+  Services.obs.removeObserver(TestObserver, "passwordmgr-storage-changed");
+  Services.logins.removeAllUserFacingLogins();
+});
+
 /**
  * Tests invalid combinations of httpRealm and formActionOrigin.
  *
@@ -116,64 +187,70 @@ add_task(function test_addLogin_removeLogin() {
  * The legacy case of an empty string in formActionOrigin and a null value in
  * httpRealm is also supported for storage at the moment.
  */
-add_task(function test_invalid_httpRealm_formActionOrigin() {
+add_task(async function test_invalid_httpRealm_formActionOrigin() {
   // httpRealm === null, formActionOrigin === null
-  checkLoginInvalid(
+  await checkLoginInvalid(
     TestData.formLogin({ formActionOrigin: null }),
     /without a httpRealm or formActionOrigin/
   );
 
   // httpRealm === "", formActionOrigin === null
-  checkLoginInvalid(
+  await checkLoginInvalid(
     TestData.authLogin({ httpRealm: "" }),
     /without a httpRealm or formActionOrigin/
   );
 
   // httpRealm === null, formActionOrigin === ""
   // TODO: This is not enforced for now.
-  // checkLoginInvalid(TestData.formLogin({ formActionOrigin: "" }),
+  // await checkLoginInvalid(TestData.formLogin({ formActionOrigin: "" }),
   //                   /without a httpRealm or formActionOrigin/);
 
   // httpRealm === "", formActionOrigin === ""
   let login = TestData.formLogin({ formActionOrigin: "" });
   login.httpRealm = "";
-  checkLoginInvalid(login, /both a httpRealm and formActionOrigin/);
+  await checkLoginInvalid(login, /both a httpRealm and formActionOrigin/);
 
   // !!httpRealm, !!formActionOrigin
   login = TestData.formLogin();
   login.httpRealm = "The HTTP Realm";
-  checkLoginInvalid(login, /both a httpRealm and formActionOrigin/);
+  await checkLoginInvalid(login, /both a httpRealm and formActionOrigin/);
 
   // httpRealm === "", !!formActionOrigin
   login = TestData.formLogin();
   login.httpRealm = "";
-  checkLoginInvalid(login, /both a httpRealm and formActionOrigin/);
+  await checkLoginInvalid(login, /both a httpRealm and formActionOrigin/);
 
   // !!httpRealm, formActionOrigin === ""
   login = TestData.authLogin();
   login.formActionOrigin = "";
-  checkLoginInvalid(login, /both a httpRealm and formActionOrigin/);
+  await checkLoginInvalid(login, /both a httpRealm and formActionOrigin/);
 });
 
 /**
  * Tests null or empty values in required login properties.
  */
-add_task(function test_missing_properties() {
-  checkLoginInvalid(
+add_task(async function test_missing_properties() {
+  await checkLoginInvalid(
     TestData.formLogin({ origin: null }),
     /null or empty origin/
   );
 
-  checkLoginInvalid(TestData.formLogin({ origin: "" }), /null or empty origin/);
+  await checkLoginInvalid(
+    TestData.formLogin({ origin: "" }),
+    /null or empty origin/
+  );
 
-  checkLoginInvalid(TestData.formLogin({ username: null }), /null username/);
+  await checkLoginInvalid(
+    TestData.formLogin({ username: null }),
+    /null username/
+  );
 
-  checkLoginInvalid(
+  await checkLoginInvalid(
     TestData.formLogin({ password: null }),
     /null or empty password/
   );
 
-  checkLoginInvalid(
+  await checkLoginInvalid(
     TestData.formLogin({ password: "" }),
     /null or empty password/
   );
@@ -182,7 +259,7 @@ add_task(function test_missing_properties() {
 /**
  * Tests invalid NUL characters in nsILoginInfo properties.
  */
-add_task(function test_invalid_characters() {
+add_task(async function test_invalid_characters() {
   let loginList = [
     TestData.authLogin({ origin: "http://null\0X.example.com" }),
     TestData.authLogin({ httpRealm: "realm\0" }),
@@ -194,7 +271,7 @@ add_task(function test_invalid_characters() {
     TestData.formLogin({ password: "pass\0word" }),
   ];
   for (let loginInfo of loginList) {
-    checkLoginInvalid(loginInfo, /login values can't contain nulls/);
+    await checkLoginInvalid(loginInfo, /login values can't contain nulls/);
   }
 });
 
@@ -211,21 +288,20 @@ add_task(function test_removeLogin_nonexisting() {
 /**
  * Tests removing all logins at once.
  */
-add_task(function test_removeAllLogins() {
-  for (let loginInfo of TestData.loginList()) {
-    Services.logins.addLogin(loginInfo);
-  }
-  Services.logins.removeAllLogins();
+add_task(async function test_removeAllUserFacingLogins() {
+  await Services.logins.addLogins(TestData.loginList());
+
+  Services.logins.removeAllUserFacingLogins();
   LoginTestUtils.checkLogins([]);
 
   // The function should also work when there are no logins to delete.
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
 });
 
 /**
  * Tests the modifyLogin function with an nsILoginInfo argument.
  */
-add_task(function test_modifyLogin_nsILoginInfo() {
+add_task(async function test_modifyLogin_nsILoginInfo() {
   let loginInfo = TestData.formLogin();
   let updatedLoginInfo = TestData.formLogin({
     username: "new username",
@@ -242,7 +318,7 @@ add_task(function test_modifyLogin_nsILoginInfo() {
   );
 
   // Add the first form login, then modify it to match the second.
-  Services.logins.addLogin(loginInfo);
+  await Services.logins.addLoginAsync(loginInfo);
   Services.logins.modifyLogin(loginInfo, updatedLoginInfo);
 
   // The data should now match the second login.
@@ -257,7 +333,7 @@ add_task(function test_modifyLogin_nsILoginInfo() {
   LoginTestUtils.checkLogins([differentLoginInfo]);
 
   // It is now possible to add a login with the old type and origin.
-  Services.logins.addLogin(loginInfo);
+  await Services.logins.addLoginAsync(loginInfo);
   LoginTestUtils.checkLogins([loginInfo, differentLoginInfo]);
 
   // Modifying a login to match an existing one should not be possible.
@@ -273,7 +349,7 @@ add_task(function test_modifyLogin_nsILoginInfo() {
 /**
  * Tests the modifyLogin function with an nsIPropertyBag argument.
  */
-add_task(function test_modifyLogin_nsIProperyBag() {
+add_task(async function test_modifyLogin_nsIProperyBag() {
   let loginInfo = TestData.formLogin();
   let updatedLoginInfo = TestData.formLogin({
     username: "new username",
@@ -300,7 +376,7 @@ add_task(function test_modifyLogin_nsIProperyBag() {
 
   // Add the first form login, then modify it to match the second, changing
   // only some of its properties and checking the behavior with an empty string.
-  Services.logins.addLogin(loginInfo);
+  await Services.logins.addLoginAsync(loginInfo);
   Services.logins.modifyLogin(
     loginInfo,
     newPropertyBag({
@@ -338,7 +414,7 @@ add_task(function test_modifyLogin_nsIProperyBag() {
   LoginTestUtils.checkLogins([differentLoginInfo]);
 
   // It is now possible to add a login with the old type and origin.
-  Services.logins.addLogin(loginInfo);
+  await Services.logins.addLoginAsync(loginInfo);
   LoginTestUtils.checkLogins([loginInfo, differentLoginInfo]);
 
   // Modifying a login to match an existing one should not be possible.
@@ -417,8 +493,9 @@ add_task(function test_deduplicate_keeps_most_recent() {
   Assert.equal(deduped.length, 1, "Deduplicated the logins array.");
 
   // Verify that the remaining login have the most recent date.
-  let loginTimeLastUsed = deduped[0].QueryInterface(Ci.nsILoginMetaInfo)
-    .timeLastUsed;
+  let loginTimeLastUsed = deduped[0].QueryInterface(
+    Ci.nsILoginMetaInfo
+  ).timeLastUsed;
   Assert.equal(
     loginTimeLastUsed,
     Date.UTC(2015, 11, 4, 0, 0, 0),
@@ -430,8 +507,9 @@ add_task(function test_deduplicate_keeps_most_recent() {
   Assert.equal(deduped.length, 1, "Deduplicated the reversed logins array.");
 
   // Verify that the remaining login have the most recent date.
-  loginTimeLastUsed = deduped[0].QueryInterface(Ci.nsILoginMetaInfo)
-    .timeLastUsed;
+  loginTimeLastUsed = deduped[0].QueryInterface(
+    Ci.nsILoginMetaInfo
+  ).timeLastUsed;
   Assert.equal(
     loginTimeLastUsed,
     Date.UTC(2015, 11, 4, 0, 0, 0),
@@ -442,7 +520,7 @@ add_task(function test_deduplicate_keeps_most_recent() {
 /**
  * Tests handling when adding a login with bad date values
  */
-add_task(function test_addLogin_badDates() {
+add_task(async function test_addLogin_badDates() {
   LoginTestUtils.clearData();
 
   let now = Date.now();
@@ -457,10 +535,10 @@ add_task(function test_addLogin_badDates() {
     Assert.ok(!defaultsLogin[pname]);
   }
   Assert.ok(
-    !!Services.logins.addLogin(defaultsLogin),
+    !!(await Services.logins.addLoginAsync(defaultsLogin)),
     "Sanity check adding defaults formLogin"
   );
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
 
   // 0 is a valid date in this context - new nsLoginInfo timestamps init to 0
   for (let pname of ["timeCreated", "timeLastUsed", "timePasswordChanged"]) {
@@ -470,10 +548,10 @@ add_task(function test_addLogin_badDates() {
       })
     );
     Assert.ok(
-      !!Services.logins.addLogin(loginInfo),
+      !!(await Services.logins.addLoginAsync(loginInfo)),
       "Check 0 value for " + pname
     );
-    Services.logins.removeAllLogins();
+    Services.logins.removeAllUserFacingLogins();
   }
 
   // negative dates get clamped to 0 and are ok
@@ -484,10 +562,10 @@ add_task(function test_addLogin_badDates() {
       })
     );
     Assert.ok(
-      !!Services.logins.addLogin(loginInfo),
+      !!(await Services.logins.addLoginAsync(loginInfo)),
       "Check -1 value for " + pname
     );
-    Services.logins.removeAllLogins();
+    Services.logins.removeAllUserFacingLogins();
   }
 
   // out-of-range dates will throw
@@ -497,8 +575,8 @@ add_task(function test_addLogin_badDates() {
         [pname]: MAX_DATE_MS + 1,
       })
     );
-    Assert.throws(
-      () => Services.logins.addLogin(loginInfo),
+    await Assert.rejects(
+      Services.logins.addLoginAsync(loginInfo),
       /invalid date properties/
     );
     Assert.equal(Services.logins.getAllLogins().length, 0);
@@ -516,33 +594,35 @@ add_task(async function test_addLogins_badDates() {
   let defaultsLogin = TestData.formLogin({
     username: "defaults",
   });
+  await Services.logins.addLoginAsync(defaultsLogin);
 
   // -11644473600000 is the value you get if you convert Dec 31 1600 16:07:02 to unix epoch time
   let timeCreatedLogin = TestData.formLogin({
     username: "tc",
     timeCreated: -11644473600000,
   });
+  await Assert.rejects(
+    Services.logins.addLoginAsync(timeCreatedLogin),
+    /Can\'t add a login with invalid date properties./
+  );
 
   let timeLastUsedLogin = TestData.formLogin({
     username: "tlu",
     timeLastUsed: -11644473600000,
   });
+  await Assert.rejects(
+    Services.logins.addLoginAsync(timeLastUsedLogin),
+    /Can\'t add a login with invalid date properties./
+  );
 
   let timePasswordChangedLogin = TestData.formLogin({
     username: "tpc",
     timePasswordChanged: -11644473600000,
   });
+  await Assert.rejects(
+    Services.logins.addLoginAsync(timePasswordChangedLogin),
+    /Can\'t add a login with invalid date properties./
+  );
 
-  await Services.logins.addLogins([
-    defaultsLogin,
-    timeCreatedLogin,
-    timeLastUsedLogin,
-    timePasswordChangedLogin,
-  ]);
-
-  // none of the logins with invalid dates should have been added
-  let savedLogins = Services.logins.getAllLogins();
-  Assert.equal(savedLogins.length, 1);
-
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
 });
