@@ -9,7 +9,6 @@
 #include "nsWeakReference.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIObserver.h"
-#include "nsIAsyncShutdown.h"
 #include "mozilla/storage.h"
 #include "mozilla/storage/StatementCache.h"
 #include "mozilla/Attributes.h"
@@ -19,7 +18,7 @@
 
 // This is the schema version. Update it at any schema change and add a
 // corresponding migrateVxx method below.
-#define DATABASE_SCHEMA_VERSION 52
+#define DATABASE_SCHEMA_VERSION 74
 
 // Fired after Places inited.
 #define TOPIC_PLACES_INIT_COMPLETE "places-init-complete"
@@ -39,10 +38,11 @@
 // calling `observe` directly on the database. Used for testing only.
 #define TOPIC_SIMULATE_PLACES_SHUTDOWN "test-simulate-places-shutdown"
 
+class mozIStorageService;
+class nsIAsyncShutdownClient;
 class nsIRunnable;
 
-namespace mozilla {
-namespace places {
+namespace mozilla::places {
 
 enum JournalMode {
   // Default SQLite journal mode.
@@ -63,9 +63,9 @@ class ClientsShutdownBlocker;
 class ConnectionShutdownBlocker;
 
 class Database final : public nsIObserver, public nsSupportsWeakReference {
-  typedef mozilla::storage::StatementCache<mozIStorageStatement> StatementCache;
-  typedef mozilla::storage::StatementCache<mozIStorageAsyncStatement>
-      AsyncStatementCache;
+  using StatementCache = mozilla::storage::StatementCache<mozIStorageStatement>;
+  using AsyncStatementCache =
+      mozilla::storage::StatementCache<mozIStorageAsyncStatement>;
 
  public:
   NS_DECL_THREADSAFE_ISUPPORTS
@@ -202,32 +202,11 @@ class Database final : public nsIObserver, public nsSupportsWeakReference {
   already_AddRefed<mozIStorageAsyncStatement> GetAsyncStatement(
       const nsACString& aQuery);
 
-  uint32_t MaxUrlLength();
-
-  int64_t GetRootFolderId() {
-    mozilla::Unused << EnsureConnection();
-    return mRootId;
-  }
-  int64_t GetMenuFolderId() {
-    mozilla::Unused << EnsureConnection();
-    return mMenuRootId;
-  }
   int64_t GetTagsFolderId() {
     mozilla::Unused << EnsureConnection();
     return mTagsRootId;
   }
-  int64_t GetUnfiledFolderId() {
-    mozilla::Unused << EnsureConnection();
-    return mUnfiledRootId;
-  }
-  int64_t GetToolbarFolderId() {
-    mozilla::Unused << EnsureConnection();
-    return mToolbarRootId;
-  }
-  int64_t GetMobileFolderId() {
-    mozilla::Unused << EnsureConnection();
-    return mMobileRootId;
-  }
+  nsresult RecalculateOriginFrecencyStatsInternal();
 
  protected:
   /**
@@ -236,8 +215,6 @@ class Database final : public nsIObserver, public nsSupportsWeakReference {
    */
   void Shutdown();
 
-  bool IsShutdownStarted() const;
-
   /**
    * Ensure the favicons database file exists.
    *
@@ -319,19 +296,6 @@ class Database final : public nsIObserver, public nsSupportsWeakReference {
   /**
    * Helpers used by schema upgrades.
    */
-  nsresult MigrateV31Up();
-  nsresult MigrateV32Up();
-  nsresult MigrateV33Up();
-  nsresult MigrateV34Up();
-  nsresult MigrateV35Up();
-  nsresult MigrateV36Up();
-  nsresult MigrateV37Up();
-  nsresult MigrateV38Up();
-  nsresult MigrateV39Up();
-  nsresult MigrateV40Up();
-  nsresult MigrateV41Up();
-  nsresult MigrateV42Up();
-  nsresult MigrateV43Up();
   nsresult MigrateV44Up();
   nsresult MigrateV45Up();
   nsresult MigrateV46Up();
@@ -341,6 +305,20 @@ class Database final : public nsIObserver, public nsSupportsWeakReference {
   nsresult MigrateV50Up();
   nsresult MigrateV51Up();
   nsresult MigrateV52Up();
+  nsresult MigrateV53Up();
+  nsresult MigrateV54Up();
+  nsresult MigrateV55Up();
+  nsresult MigrateV56Up();
+  nsresult MigrateV57Up();
+  nsresult MigrateV60Up();
+  nsresult MigrateV61Up();
+  nsresult MigrateV67Up();
+  nsresult MigrateV69Up();
+  nsresult MigrateV70Up();
+  nsresult MigrateV71Up();
+  nsresult MigrateV72Up();
+  nsresult MigrateV73Up();
+  nsresult MigrateV74Up();
 
   void MigrateV52OriginFrecencies();
 
@@ -350,9 +328,6 @@ class Database final : public nsIObserver, public nsSupportsWeakReference {
 
   int64_t CreateMobileRoot();
   nsresult ConvertOldStyleQuery(nsCString& aURL);
-  nsresult GetItemsWithAnno(const nsACString& aAnnoName, int32_t aItemType,
-                            nsTArray<int64_t>& aItemIds);
-  nsresult DeleteBookmarkItem(int32_t aItemId);
 
  private:
   ~Database();
@@ -373,12 +348,6 @@ class Database final : public nsIObserver, public nsSupportsWeakReference {
   int32_t mDBPageSize;
   uint16_t mDatabaseStatus;
   bool mClosed;
-  // Used to track whether icon payloads should be converted at the end of
-  // schema migration.
-  bool mShouldConvertIconPayloads;
-  // Used to track whether the favicons database should be vacuumed at the end
-  // of the schema migration.
-  bool mShouldVacuumIcons;
 
   /**
    * Phases for shutting down the Database.
@@ -415,7 +384,6 @@ class Database final : public nsIObserver, public nsSupportsWeakReference {
   int64_t mMobileRootId;
 };
 
-}  // namespace places
-}  // namespace mozilla
+}  // namespace mozilla::places
 
 #endif  // mozilla_places_Database_h_
