@@ -1,11 +1,9 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "Preferences",
-  "resource://gre/modules/Preferences.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  Preferences: "resource://gre/modules/Preferences.sys.mjs",
+});
 
 add_task(async function insert_separator_notification() {
   let observer = expectPlacesObserverNotifications(["bookmark-added"]);
@@ -150,15 +148,16 @@ add_task(async function insert_bookmark_tag_notification() {
     url: new URL("http://tag.example.com/"),
   });
   let itemId = await PlacesUtils.promiseItemId(bm.guid);
-  let parentId = await PlacesUtils.promiseItemId(bm.parentGuid);
 
   let tagFolder = await PlacesUtils.bookmarks.insert({
     type: PlacesUtils.bookmarks.TYPE_FOLDER,
     parentGuid: PlacesUtils.bookmarks.tagsGuid,
     title: "tag",
   });
-  let placesObserver = expectPlacesObserverNotifications(["bookmark-added"]);
-  let bookmarksObserver = expectNotifications();
+  const observer = expectPlacesObserverNotifications([
+    "bookmark-added",
+    "bookmark-tags-changed",
+  ]);
   let tag = await PlacesUtils.bookmarks.insert({
     type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
     parentGuid: tagFolder.guid,
@@ -167,7 +166,7 @@ add_task(async function insert_bookmark_tag_notification() {
   let tagId = await PlacesUtils.promiseItemId(tag.guid);
   let tagParentId = await PlacesUtils.promiseItemId(tag.parentGuid);
 
-  placesObserver.check([
+  observer.check([
     {
       type: "bookmark-added",
       id: tagId,
@@ -182,24 +181,17 @@ add_task(async function insert_bookmark_tag_notification() {
       source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
       isTagging: true,
     },
-  ]);
-
-  bookmarksObserver.check([
     {
-      name: "onItemChanged",
-      arguments: [
-        itemId,
-        "tags",
-        false,
-        "",
-        PlacesUtils.toPRTime(bm.lastModified),
-        bm.type,
-        parentId,
-        bm.guid,
-        bm.parentGuid,
-        "",
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-tags-changed",
+      id: itemId,
+      itemType: bm.type,
+      url: bm.url,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      tags: ["tag"],
+      lastModified: bm.lastModified,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      isTagging: false,
     },
   ]);
 });
@@ -208,7 +200,7 @@ add_task(async function update_bookmark_lastModified() {
   let timerPrecision = Preferences.get("privacy.reduceTimerPrecision");
   Preferences.set("privacy.reduceTimerPrecision", false);
 
-  registerCleanupFunction(function() {
+  registerCleanupFunction(function () {
     Preferences.set("privacy.reduceTimerPrecision", timerPrecision);
   });
 
@@ -217,30 +209,25 @@ add_task(async function update_bookmark_lastModified() {
     parentGuid: PlacesUtils.bookmarks.unfiledGuid,
     url: new URL("http://lastmod.example.com/"),
   });
-  let observer = expectNotifications();
+  const observer = expectPlacesObserverNotifications(["bookmark-time-changed"]);
   bm = await PlacesUtils.bookmarks.update({
     guid: bm.guid,
     lastModified: new Date(),
   });
   let itemId = await PlacesUtils.promiseItemId(bm.guid);
-  let parentId = await PlacesUtils.promiseItemId(bm.parentGuid);
 
   observer.check([
     {
-      name: "onItemChanged",
-      arguments: [
-        itemId,
-        "lastModified",
-        false,
-        `${PlacesUtils.toPRTime(bm.lastModified)}`,
-        PlacesUtils.toPRTime(bm.lastModified),
-        bm.type,
-        parentId,
-        bm.guid,
-        bm.parentGuid,
-        "",
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-time-changed",
+      id: itemId,
+      itemType: bm.type,
+      url: bm.url,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      dateAdded: bm.dateAdded,
+      lastModified: bm.lastModified,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      isTagging: false,
     },
   ]);
 });
@@ -251,30 +238,27 @@ add_task(async function update_bookmark_title() {
     parentGuid: PlacesUtils.bookmarks.unfiledGuid,
     url: new URL("http://title.example.com/"),
   });
-  let observer = expectNotifications();
+  const observer = expectPlacesObserverNotifications([
+    "bookmark-title-changed",
+  ]);
   bm = await PlacesUtils.bookmarks.update({
     guid: bm.guid,
     title: "new title",
   });
   let itemId = await PlacesUtils.promiseItemId(bm.guid);
-  let parentId = await PlacesUtils.promiseItemId(bm.parentGuid);
 
   observer.check([
     {
-      name: "onItemChanged",
-      arguments: [
-        itemId,
-        "title",
-        false,
-        bm.title,
-        PlacesUtils.toPRTime(bm.lastModified),
-        bm.type,
-        parentId,
-        bm.guid,
-        bm.parentGuid,
-        "",
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-title-changed",
+      id: itemId,
+      itemType: bm.type,
+      url: bm.url,
+      title: bm.title,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      lastModified: bm.lastModified,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      isTagging: false,
     },
   ]);
 });
@@ -285,30 +269,24 @@ add_task(async function update_bookmark_uri() {
     parentGuid: PlacesUtils.bookmarks.unfiledGuid,
     url: new URL("http://url.example.com/"),
   });
-  let observer = expectNotifications();
+  const observer = expectPlacesObserverNotifications(["bookmark-url-changed"]);
   bm = await PlacesUtils.bookmarks.update({
     guid: bm.guid,
     url: "http://mozilla.org/",
   });
   let itemId = await PlacesUtils.promiseItemId(bm.guid);
-  let parentId = await PlacesUtils.promiseItemId(bm.parentGuid);
 
   observer.check([
     {
-      name: "onItemChanged",
-      arguments: [
-        itemId,
-        "uri",
-        false,
-        bm.url.href,
-        PlacesUtils.toPRTime(bm.lastModified),
-        bm.type,
-        parentId,
-        bm.guid,
-        bm.parentGuid,
-        "http://url.example.com/",
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-url-changed",
+      id: itemId,
+      itemType: bm.type,
+      url: bm.url.href,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      isTagging: false,
+      lastModified: bm.lastModified,
     },
   ]);
 });
@@ -326,10 +304,10 @@ add_task(async function update_move_same_folder() {
     url: new URL("http://move.example.com/"),
   });
   let bmItemId = await PlacesUtils.promiseItemId(bm.guid);
-  let bmParentId = await PlacesUtils.promiseItemId(bm.parentGuid);
   let bmOldIndex = bm.index;
 
-  let observer = expectNotifications();
+  let observer = expectPlacesObserverNotifications(["bookmark-moved"]);
+
   bm = await PlacesUtils.bookmarks.update({
     guid: bm.guid,
     parentGuid: PlacesUtils.bookmarks.unfiledGuid,
@@ -338,26 +316,23 @@ add_task(async function update_move_same_folder() {
   Assert.equal(bm.index, 0);
   observer.check([
     {
-      name: "onItemMoved",
-      arguments: [
-        bmItemId,
-        bmParentId,
-        bmOldIndex,
-        bmParentId,
-        bm.index,
-        bm.type,
-        bm.guid,
-        bm.parentGuid,
-        bm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-        "http://move.example.com/",
-      ],
+      type: "bookmark-moved",
+      id: bmItemId,
+      itemType: bm.type,
+      url: "http://move.example.com/",
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      index: bm.index,
+      oldParentGuid: bm.parentGuid,
+      oldIndex: bmOldIndex,
+      isTagging: false,
     },
   ]);
 
   // Test that we get the right index for DEFAULT_INDEX input.
   bmOldIndex = 0;
-  observer = expectNotifications();
+  observer = expectPlacesObserverNotifications(["bookmark-moved"]);
   bm = await PlacesUtils.bookmarks.update({
     guid: bm.guid,
     parentGuid: PlacesUtils.bookmarks.unfiledGuid,
@@ -366,20 +341,17 @@ add_task(async function update_move_same_folder() {
   Assert.ok(bm.index > 0);
   observer.check([
     {
-      name: "onItemMoved",
-      arguments: [
-        bmItemId,
-        bmParentId,
-        bmOldIndex,
-        bmParentId,
-        bm.index,
-        bm.type,
-        bm.guid,
-        bm.parentGuid,
-        bm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-        bm.url,
-      ],
+      type: "bookmark-moved",
+      id: bmItemId,
+      itemType: bm.type,
+      url: bm.url,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      index: bm.index,
+      oldParentGuid: bm.parentGuid,
+      oldIndex: bmOldIndex,
+      isTagging: false,
     },
   ]);
 });
@@ -395,35 +367,66 @@ add_task(async function update_move_different_folder() {
     parentGuid: PlacesUtils.bookmarks.unfiledGuid,
   });
   let bmItemId = await PlacesUtils.promiseItemId(bm.guid);
-  let bmOldParentId = await PlacesUtils.promiseItemId(
-    PlacesUtils.bookmarks.unfiledGuid
-  );
   let bmOldIndex = bm.index;
 
-  let observer = expectNotifications();
+  const observer = expectPlacesObserverNotifications(["bookmark-moved"]);
   bm = await PlacesUtils.bookmarks.update({
     guid: bm.guid,
     parentGuid: folder.guid,
     index: PlacesUtils.bookmarks.DEFAULT_INDEX,
   });
   Assert.equal(bm.index, 0);
-  let bmNewParentId = await PlacesUtils.promiseItemId(folder.guid);
   observer.check([
     {
-      name: "onItemMoved",
-      arguments: [
-        bmItemId,
-        bmOldParentId,
-        bmOldIndex,
-        bmNewParentId,
-        bm.index,
-        bm.type,
-        bm.guid,
-        PlacesUtils.bookmarks.unfiledGuid,
-        bm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-        "http://move.example.com/",
-      ],
+      type: "bookmark-moved",
+      id: bmItemId,
+      itemType: bm.type,
+      url: "http://move.example.com/",
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      index: bm.index,
+      oldParentGuid: PlacesUtils.bookmarks.unfiledGuid,
+      oldIndex: bmOldIndex,
+      isTagging: false,
+    },
+  ]);
+});
+
+add_task(async function update_move_tag_folder() {
+  let bm = await PlacesUtils.bookmarks.insert({
+    type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+    parentGuid: PlacesUtils.bookmarks.unfiledGuid,
+    url: new URL("http://move.example.com/"),
+  });
+  let folder = await PlacesUtils.bookmarks.insert({
+    type: PlacesUtils.bookmarks.TYPE_FOLDER,
+    parentGuid: PlacesUtils.bookmarks.tagsGuid,
+    title: "tag",
+  });
+  let bmItemId = await PlacesUtils.promiseItemId(bm.guid);
+  let bmOldIndex = bm.index;
+
+  const observer = expectPlacesObserverNotifications(["bookmark-moved"]);
+  bm = await PlacesUtils.bookmarks.update({
+    guid: bm.guid,
+    parentGuid: folder.guid,
+    index: PlacesUtils.bookmarks.DEFAULT_INDEX,
+  });
+  Assert.equal(bm.index, 0);
+  observer.check([
+    {
+      type: "bookmark-moved",
+      id: bmItemId,
+      itemType: bm.type,
+      url: "http://move.example.com/",
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      index: bm.index,
+      oldParentGuid: PlacesUtils.bookmarks.unfiledGuid,
+      oldIndex: bmOldIndex,
+      isTagging: true,
     },
   ]);
 });
@@ -437,21 +440,20 @@ add_task(async function remove_bookmark() {
   let itemId = await PlacesUtils.promiseItemId(bm.guid);
   let parentId = await PlacesUtils.promiseItemId(bm.parentGuid);
 
-  let observer = expectNotifications();
+  let observer = expectPlacesObserverNotifications(["bookmark-removed"]);
   await PlacesUtils.bookmarks.remove(bm.guid);
   observer.check([
     {
-      name: "onItemRemoved",
-      arguments: [
-        itemId,
-        parentId,
-        bm.index,
-        bm.type,
-        bm.url,
-        bm.guid,
-        bm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: itemId,
+      parentId,
+      index: bm.index,
+      url: bm.url,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: false,
     },
   ]);
 });
@@ -472,34 +474,32 @@ add_task(async function remove_multiple_bookmarks() {
   let itemId2 = await PlacesUtils.promiseItemId(bm2.guid);
   let parentId2 = await PlacesUtils.promiseItemId(bm2.parentGuid);
 
-  let observer = expectNotifications();
+  let observer = expectPlacesObserverNotifications(["bookmark-removed"]);
   await PlacesUtils.bookmarks.remove([bm1, bm2]);
   observer.check([
     {
-      name: "onItemRemoved",
-      arguments: [
-        itemId1,
-        parentId1,
-        bm1.index,
-        bm1.type,
-        bm1.url,
-        bm1.guid,
-        bm1.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: itemId1,
+      parentId: parentId1,
+      index: bm1.index,
+      url: bm1.url,
+      guid: bm1.guid,
+      parentGuid: bm1.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        itemId2,
-        parentId2,
-        bm2.index,
-        bm2.type,
-        bm2.url,
-        bm2.guid,
-        bm2.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: itemId2,
+      parentId: parentId2,
+      index: bm2.index - 1,
+      url: bm2.url,
+      guid: bm2.guid,
+      parentGuid: bm2.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: false,
     },
   ]);
 });
@@ -512,21 +512,20 @@ add_task(async function remove_folder() {
   let itemId = await PlacesUtils.promiseItemId(bm.guid);
   let parentId = await PlacesUtils.promiseItemId(bm.parentGuid);
 
-  let observer = expectNotifications();
+  let observer = expectPlacesObserverNotifications(["bookmark-removed"]);
   await PlacesUtils.bookmarks.remove(bm.guid);
   observer.check([
     {
-      name: "onItemRemoved",
-      arguments: [
-        itemId,
-        parentId,
-        bm.index,
-        bm.type,
-        null,
-        bm.guid,
-        bm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: itemId,
+      parentId,
+      index: bm.index,
+      url: null,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_FOLDER,
+      isTagging: false,
     },
   ]);
 });
@@ -538,7 +537,6 @@ add_task(async function remove_bookmark_tag_notification() {
     url: new URL("http://untag.example.com/"),
   });
   let itemId = await PlacesUtils.promiseItemId(bm.guid);
-  let parentId = await PlacesUtils.promiseItemId(bm.parentGuid);
 
   let tagFolder = await PlacesUtils.bookmarks.insert({
     type: PlacesUtils.bookmarks.TYPE_FOLDER,
@@ -553,38 +551,93 @@ add_task(async function remove_bookmark_tag_notification() {
   let tagId = await PlacesUtils.promiseItemId(tag.guid);
   let tagParentId = await PlacesUtils.promiseItemId(tag.parentGuid);
 
-  let observer = expectNotifications();
+  const observer = expectPlacesObserverNotifications([
+    "bookmark-removed",
+    "bookmark-tags-changed",
+  ]);
   await PlacesUtils.bookmarks.remove(tag.guid);
 
   observer.check([
     {
-      name: "onItemRemoved",
-      arguments: [
-        tagId,
-        tagParentId,
-        tag.index,
-        tag.type,
-        tag.url,
-        tag.guid,
-        tag.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: tagId,
+      parentId: tagParentId,
+      index: tag.index,
+      url: tag.url,
+      guid: tag.guid,
+      parentGuid: tag.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: true,
     },
     {
-      name: "onItemChanged",
-      arguments: [
-        itemId,
-        "tags",
-        false,
-        "",
-        PlacesUtils.toPRTime(bm.lastModified),
-        bm.type,
-        parentId,
-        bm.guid,
-        bm.parentGuid,
-        "",
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-tags-changed",
+      id: itemId,
+      itemType: bm.type,
+      url: bm.url,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      tags: [],
+      lastModified: bm.lastModified,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      isTagging: false,
+    },
+  ]);
+});
+
+add_task(async function rename_bookmark_tag_notification() {
+  let bm = await PlacesUtils.bookmarks.insert({
+    type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+    parentGuid: PlacesUtils.bookmarks.unfiledGuid,
+    url: new URL("http://renametag.example.com/"),
+  });
+  let itemId = await PlacesUtils.promiseItemId(bm.guid);
+
+  let tagFolder = await PlacesUtils.bookmarks.insert({
+    type: PlacesUtils.bookmarks.TYPE_FOLDER,
+    parentGuid: PlacesUtils.bookmarks.tagsGuid,
+    title: "tag",
+  });
+  let tag = await PlacesUtils.bookmarks.insert({
+    type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+    parentGuid: tagFolder.guid,
+    url: new URL("http://renametag.example.com/"),
+  });
+  let tagParentId = await PlacesUtils.promiseItemId(tag.parentGuid);
+
+  const observer = expectPlacesObserverNotifications([
+    "bookmark-title-changed",
+    "bookmark-tags-changed",
+  ]);
+  tagFolder = await PlacesUtils.bookmarks.update({
+    guid: tagFolder.guid,
+    title: "renamed",
+  });
+
+  observer.check([
+    {
+      type: "bookmark-title-changed",
+      id: tagParentId,
+      title: "renamed",
+      guid: tagFolder.guid,
+      url: "",
+      lastModified: tagFolder.lastModified,
+      parentGuid: tagFolder.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_FOLDER,
+      isTagging: true,
+    },
+    {
+      type: "bookmark-tags-changed",
+      id: itemId,
+      itemType: bm.type,
+      url: bm.url,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      tags: ["renamed"],
+      lastModified: bm.lastModified,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      isTagging: false,
     },
   ]);
 });
@@ -617,65 +670,149 @@ add_task(async function remove_folder_notification() {
   });
   let bm2ItemId = await PlacesUtils.promiseItemId(bm2.guid);
 
-  let observer = expectNotifications();
+  let observer = expectPlacesObserverNotifications(["bookmark-removed"]);
   await PlacesUtils.bookmarks.remove(folder1.guid);
 
   observer.check([
     {
-      name: "onItemRemoved",
-      arguments: [
-        bm2ItemId,
-        folder2Id,
-        bm2.index,
-        bm2.type,
-        bm2.url,
-        bm2.guid,
-        bm2.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: bm2ItemId,
+      parentId: folder2Id,
+      index: bm2.index,
+      url: bm2.url,
+      guid: bm2.guid,
+      parentGuid: bm2.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        folder2Id,
-        folder1Id,
-        folder2.index,
-        folder2.type,
-        null,
-        folder2.guid,
-        folder2.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: folder2Id,
+      parentId: folder1Id,
+      index: folder2.index,
+      url: null,
+      guid: folder2.guid,
+      parentGuid: folder2.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_FOLDER,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        bmItemId,
-        folder1Id,
-        bm.index,
-        bm.type,
-        bm.url,
-        bm.guid,
-        bm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: bmItemId,
+      parentId: folder1Id,
+      index: bm.index,
+      url: bm.url,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        folder1Id,
-        folder1ParentId,
-        folder1.index,
-        folder1.type,
-        null,
-        folder1.guid,
-        folder1.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: folder1Id,
+      parentId: folder1ParentId,
+      index: folder1.index,
+      url: null,
+      guid: folder1.guid,
+      parentGuid: folder1.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_FOLDER,
+      isTagging: false,
     },
   ]);
 });
 
+add_task(async function multiple_tags() {
+  const BOOKMARK_URL = "http://multipletags.example.com/";
+  const TAG_NAMES = ["tag1", "tag2", "tag3", "tag4", "tag5", "tag6"];
+
+  const bm = await PlacesUtils.bookmarks.insert({
+    type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+    parentGuid: PlacesUtils.bookmarks.unfiledGuid,
+    url: new URL(BOOKMARK_URL),
+  });
+  const itemId = await PlacesUtils.promiseItemId(bm.guid);
+
+  info("Register all tags");
+  const tagFolders = await Promise.all(
+    TAG_NAMES.map(tagName =>
+      PlacesUtils.bookmarks.insert({
+        type: PlacesUtils.bookmarks.TYPE_FOLDER,
+        parentGuid: PlacesUtils.bookmarks.tagsGuid,
+        title: tagName,
+      })
+    )
+  );
+
+  info("Test adding tags to bookmark");
+  for (let i = 0; i < tagFolders.length; i++) {
+    const tagFolder = tagFolders[i];
+    const expectedTagNames = TAG_NAMES.slice(0, i + 1);
+
+    const observer = expectPlacesObserverNotifications([
+      "bookmark-tags-changed",
+    ]);
+
+    await PlacesUtils.bookmarks.insert({
+      type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      parentGuid: tagFolder.guid,
+      url: new URL(BOOKMARK_URL),
+    });
+
+    observer.check([
+      {
+        type: "bookmark-tags-changed",
+        id: itemId,
+        itemType: bm.type,
+        url: bm.url,
+        guid: bm.guid,
+        parentGuid: bm.parentGuid,
+        tags: expectedTagNames,
+        lastModified: bm.lastModified,
+        source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+        isTagging: false,
+      },
+    ]);
+  }
+
+  info("Test removing tags from bookmark");
+  for (const removedLength of [1, 2, 3]) {
+    const removedTags = tagFolders.splice(0, removedLength);
+
+    const observer = expectPlacesObserverNotifications([
+      "bookmark-tags-changed",
+    ]);
+
+    // We can remove multiple tags at one time.
+    await PlacesUtils.bookmarks.remove(removedTags);
+
+    const expectedResults = [];
+
+    for (let i = 0; i < removedLength; i++) {
+      TAG_NAMES.splice(0, 1);
+      const expectedTagNames = [...TAG_NAMES];
+
+      expectedResults.push({
+        type: "bookmark-tags-changed",
+        id: itemId,
+        itemType: bm.type,
+        url: bm.url,
+        guid: bm.guid,
+        parentGuid: bm.parentGuid,
+        tags: expectedTagNames,
+        lastModified: bm.lastModified,
+        source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+        isTagging: false,
+      });
+    }
+
+    observer.check(expectedResults);
+  }
+});
+
 add_task(async function eraseEverything_notification() {
   // Let's start from a clean situation.
   await PlacesUtils.bookmarks.eraseEverything();
@@ -718,75 +855,70 @@ add_task(async function eraseEverything_notification() {
   let menuBmId = await PlacesUtils.promiseItemId(menuBm.guid);
   let menuBmParentId = await PlacesUtils.promiseItemId(menuBm.parentGuid);
 
-  let observer = expectNotifications();
+  let observer = expectPlacesObserverNotifications(["bookmark-removed"]);
   await PlacesUtils.bookmarks.eraseEverything();
 
   // Bookmarks should always be notified before their parents.
   observer.check([
     {
-      name: "onItemRemoved",
-      arguments: [
-        itemId,
-        parentId,
-        bm.index,
-        bm.type,
-        bm.url,
-        bm.guid,
-        bm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: itemId,
+      parentId,
+      index: bm.index,
+      url: bm.url,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        folder2Id,
-        folder2ParentId,
-        folder2.index,
-        folder2.type,
-        null,
-        folder2.guid,
-        folder2.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: folder2Id,
+      parentId: folder2ParentId,
+      index: folder2.index,
+      url: null,
+      guid: folder2.guid,
+      parentGuid: folder2.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_FOLDER,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        folder1Id,
-        folder1ParentId,
-        folder1.index,
-        folder1.type,
-        null,
-        folder1.guid,
-        folder1.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: folder1Id,
+      parentId: folder1ParentId,
+      index: folder1.index,
+      url: null,
+      guid: folder1.guid,
+      parentGuid: folder1.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_FOLDER,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        menuBmId,
-        menuBmParentId,
-        menuBm.index,
-        menuBm.type,
-        menuBm.url,
-        menuBm.guid,
-        menuBm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: menuBmId,
+      parentId: menuBmParentId,
+      index: menuBm.index,
+      url: menuBm.url,
+      guid: menuBm.guid,
+      parentGuid: menuBm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        toolbarBmId,
-        toolbarBmParentId,
-        toolbarBm.index,
-        toolbarBm.type,
-        toolbarBm.url,
-        toolbarBm.guid,
-        toolbarBm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: toolbarBmId,
+      parentId: toolbarBmParentId,
+      index: toolbarBm.index,
+      url: toolbarBm.url,
+      guid: toolbarBm.guid,
+      parentGuid: toolbarBm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: false,
     },
   ]);
 });
@@ -820,49 +952,46 @@ add_task(async function eraseEverything_reparented_notification() {
   bm = await PlacesUtils.bookmarks.update(bm);
   let parentId = await PlacesUtils.promiseItemId(bm.parentGuid);
 
-  let observer = expectNotifications();
+  let observer = expectPlacesObserverNotifications(["bookmark-removed"]);
   await PlacesUtils.bookmarks.eraseEverything();
 
   // Bookmarks should always be notified before their parents.
   observer.check([
     {
-      name: "onItemRemoved",
-      arguments: [
-        itemId,
-        parentId,
-        bm.index,
-        bm.type,
-        bm.url,
-        bm.guid,
-        bm.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: itemId,
+      parentId,
+      index: bm.index,
+      url: bm.url,
+      guid: bm.guid,
+      parentGuid: bm.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        folder2Id,
-        folder2ParentId,
-        folder2.index,
-        folder2.type,
-        null,
-        folder2.guid,
-        folder2.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: folder2Id,
+      parentId: folder2ParentId,
+      index: folder2.index,
+      url: null,
+      guid: folder2.guid,
+      parentGuid: folder2.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_FOLDER,
+      isTagging: false,
     },
     {
-      name: "onItemRemoved",
-      arguments: [
-        folder1Id,
-        folder1ParentId,
-        folder1.index,
-        folder1.type,
-        null,
-        folder1.guid,
-        folder1.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-      ],
+      type: "bookmark-removed",
+      id: folder1Id,
+      parentId: folder1ParentId,
+      index: folder1.index,
+      url: null,
+      guid: folder1.guid,
+      parentGuid: folder1.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      itemType: PlacesUtils.bookmarks.TYPE_FOLDER,
+      isTagging: false,
     },
   ]);
 });
@@ -900,44 +1029,47 @@ add_task(async function reorder_notification() {
 
   // Randomly reorder the array.
   sorted.sort(() => 0.5 - Math.random());
+  // Ensure there's at least one item out of place, since random does not
+  // necessarily mean they are unordered.
+  if (sorted[0].url == bookmarks[0].url) {
+    sorted.push(sorted.shift());
+  }
 
-  let observer = expectNotifications();
+  const observer = expectPlacesObserverNotifications(["bookmark-moved"]);
   await PlacesUtils.bookmarks.reorder(
     PlacesUtils.bookmarks.unfiledGuid,
     sorted.map(bm => bm.guid)
   );
 
   let expectedNotifications = [];
-  let unfiledBookmarksFolderId = await PlacesUtils.promiseItemId(
-    PlacesUtils.bookmarks.unfiledGuid
-  );
   for (let i = 0; i < sorted.length; ++i) {
     let child = sorted[i];
     let childId = await PlacesUtils.promiseItemId(child.guid);
     expectedNotifications.push({
-      name: "onItemMoved",
-      arguments: [
-        childId,
-        unfiledBookmarksFolderId,
-        child.index,
-        unfiledBookmarksFolderId,
-        i,
-        child.type,
-        child.guid,
-        child.parentGuid,
-        child.parentGuid,
-        Ci.nsINavBookmarksService.SOURCE_DEFAULT,
-        child.url,
-      ],
+      type: "bookmark-moved",
+      id: childId,
+      itemType: child.type,
+      url: child.url || "",
+      guid: child.guid,
+      parentGuid: child.parentGuid,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      index: i,
+      oldParentGuid: child.parentGuid,
+      oldIndex: child.index,
+      isTagging: false,
     });
   }
+
   observer.check(expectedNotifications);
 });
 
 add_task(async function update_notitle_notification() {
   let toolbarBmURI = Services.io.newURI("https://example.com");
+  let toolbarItemId = await PlacesUtils.promiseItemId(
+    PlacesUtils.bookmarks.toolbarGuid
+  );
   let toolbarBmId = PlacesUtils.bookmarks.insertBookmark(
-    PlacesUtils.toolbarFolderId,
+    toolbarItemId,
     toolbarBmURI,
     0,
     "Bookmark"
@@ -952,7 +1084,9 @@ add_task(async function update_notitle_notification() {
   });
   let menuFolderId = await PlacesUtils.promiseItemId(menuFolder.guid);
 
-  let observer = expectNotifications();
+  const observer = expectPlacesObserverNotifications([
+    "bookmark-title-changed",
+  ]);
 
   PlacesUtils.bookmarks.setItemTitle(toolbarBmId, null);
   strictEqual(
@@ -974,68 +1108,28 @@ add_task(async function update_notitle_notification() {
   let toolbarBmModified = await PlacesUtils.bookmarks.fetch(toolbarBmGuid);
   observer.check([
     {
-      name: "onItemChanged",
-      arguments: [
-        toolbarBmId,
-        "title",
-        false,
-        "",
-        PlacesUtils.toPRTime(toolbarBmModified.lastModified),
-        PlacesUtils.bookmarks.TYPE_BOOKMARK,
-        PlacesUtils.toolbarFolderId,
-        toolbarBmGuid,
-        PlacesUtils.bookmarks.toolbarGuid,
-        "",
-        PlacesUtils.bookmarks.SOURCES.DEFAULT,
-      ],
+      type: "bookmark-title-changed",
+      id: toolbarBmId,
+      itemType: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+      url: toolbarBmURI.spec,
+      title: "",
+      guid: toolbarBmGuid,
+      parentGuid: PlacesUtils.bookmarks.toolbarGuid,
+      lastModified: toolbarBmModified.lastModified,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      isTagging: false,
     },
     {
-      name: "onItemChanged",
-      arguments: [
-        menuFolderId,
-        "title",
-        false,
-        "",
-        PlacesUtils.toPRTime(updatedMenuBm.lastModified),
-        PlacesUtils.bookmarks.TYPE_FOLDER,
-        PlacesUtils.bookmarksMenuFolderId,
-        menuFolder.guid,
-        PlacesUtils.bookmarks.menuGuid,
-        "",
-        PlacesUtils.bookmarks.SOURCES.DEFAULT,
-      ],
+      type: "bookmark-title-changed",
+      id: menuFolderId,
+      itemType: PlacesUtils.bookmarks.TYPE_FOLDER,
+      url: "",
+      title: "",
+      guid: menuFolder.guid,
+      parentGuid: PlacesUtils.bookmarks.menuGuid,
+      lastModified: updatedMenuBm.lastModified,
+      source: Ci.nsINavBookmarksService.SOURCE_DEFAULT,
+      isTagging: false,
     },
   ]);
 });
-
-function expectNotifications() {
-  let notifications = [];
-  let observer = new Proxy(NavBookmarkObserver, {
-    get(target, name) {
-      if (name == "check") {
-        PlacesUtils.bookmarks.removeObserver(observer);
-        return expectedNotifications =>
-          Assert.deepEqual(notifications, expectedNotifications);
-      }
-
-      if (name.startsWith("onItem")) {
-        return (...origArgs) => {
-          let args = Array.from(origArgs, arg => {
-            if (arg && arg instanceof Ci.nsIURI) {
-              return new URL(arg.spec);
-            }
-            return arg;
-          });
-          notifications.push({ name, arguments: args });
-        };
-      }
-
-      if (name in target) {
-        return target[name];
-      }
-      return undefined;
-    },
-  });
-  PlacesUtils.bookmarks.addObserver(observer);
-  return observer;
-}
