@@ -5,14 +5,15 @@ async function promiseAllURLFrecencies() {
   let frecencies = new Map();
   let db = await PlacesUtils.promiseDBConnection();
   let rows = await db.execute(`
-    SELECT url, frecency
+    SELECT url, frecency, recalc_frecency
     FROM moz_places
     WHERE url_hash BETWEEN hash('http', 'prefix_lo') AND
                            hash('http', 'prefix_hi')`);
   for (let row of rows) {
-    let url = row.getResultByName("url");
-    let frecency = row.getResultByName("frecency");
-    frecencies.set(url, frecency);
+    frecencies.set(row.getResultByName("url"), {
+      frecency: row.getResultByName("frecency"),
+      recalc: row.getResultByName("recalc_frecency"),
+    });
   }
   return frecencies;
 }
@@ -106,13 +107,7 @@ add_task(async function test_update_frecencies() {
   });
 
   info("Calculate frecencies for all local URLs");
-  await PlacesUtils.withConnectionWrapper(
-    "Update all frecencies",
-    async function(db) {
-      await db.execute(`UPDATE moz_places SET
-        frecency = CALCULATE_FRECENCY(id)`);
-    }
-  );
+  await PlacesFrecencyRecalculator.recalculateAnyOutdatedFrecencies();
 
   info("Make remote changes");
   await storeRecords(buf, [
@@ -161,7 +156,7 @@ add_task(async function test_update_frecencies() {
       id: "bookmarkDDDD",
       parentid: "unfiled",
       type: "bookmark",
-      title: "D",
+      title: null,
       bmkUri: "http://example.com/d",
     },
     {
@@ -183,15 +178,14 @@ add_task(async function test_update_frecencies() {
   ]);
 
   info("Apply new items and recalculate 3 frecencies");
-  await buf.apply({
-    maxFrecenciesToRecalculate: 3,
-  });
+  await buf.apply();
+  await PlacesFrecencyRecalculator.recalculateSomeFrecencies({ chunkSize: 3 });
 
   {
     let frecencies = await promiseAllURLFrecencies();
     let urlsWithFrecency = mapFilterIterator(
       frecencies.entries(),
-      ([href, frecency]) => (frecency > 0 ? href : null)
+      ([href, { frecency, recalc }]) => (recalc == 0 ? href : null)
     );
 
     // A is unchanged, and we should recalculate frecency for three more
@@ -236,12 +230,13 @@ add_task(async function test_update_frecencies() {
 
   info("Apply new item and recalculate remaining frecencies");
   await buf.apply();
+  await PlacesFrecencyRecalculator.recalculateAnyOutdatedFrecencies();
 
   {
     let frecencies = await promiseAllURLFrecencies();
     let urlsWithoutFrecency = mapFilterIterator(
       frecencies.entries(),
-      ([href, frecency]) => (frecency <= 0 ? href : null)
+      ([href, { frecency, recalc }]) => (recalc == 1 ? href : null)
     );
     deepEqual(
       urlsWithoutFrecency,
@@ -286,7 +281,7 @@ async function setupLocalTree(localTimeSeconds) {
       },
       {
         guid: "bookmarkDDDD",
-        title: "D",
+        title: null,
         url: "http://example.com/d",
         dateAdded,
         lastModified,
@@ -381,8 +376,8 @@ add_task(async function test_apply_then_revert() {
   });
   deepEqual(
     await buf.fetchUnmergedGuids(),
-    [],
-    "Should merge all items, first time"
+    [PlacesUtils.bookmarks.menuGuid],
+    "Should leave menu with new remote structure unmerged after first time"
   );
 
   info("Revert local tree");
@@ -411,11 +406,12 @@ add_task(async function test_apply_then_revert() {
   let secondTimeRecords = await buf.apply({
     localTimeSeconds,
     remoteTimeSeconds: now,
+    notifyInStableOrder: true,
   });
   deepEqual(
     await buf.fetchUnmergedGuids(),
-    [],
-    "Should merge all items, second time"
+    [PlacesUtils.bookmarks.menuGuid],
+    "Should leave menu with new remote structure unmerged after second time"
   );
   deepEqual(
     secondTimeRecords,
@@ -429,42 +425,40 @@ add_task(async function test_apply_then_revert() {
     "folderAAAAAA",
     "bookmarkCCCC",
     "bookmarkBBBB",
+    PlacesUtils.bookmarks.menuGuid,
   ]);
-
   observer.check([
     {
-      name: "onItemRemoved",
+      name: "bookmark-removed",
       params: {
         itemId: localIdForD,
-        parentId: PlacesUtils.bookmarksMenuFolderId,
+        parentId: localItemIds.get(PlacesUtils.bookmarks.menuGuid),
         index: 1,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         urlHref: "http://example.com/d",
+        title: "", // null titles get turned into empty strings.
         guid: "bookmarkDDDD",
         parentGuid: PlacesUtils.bookmarks.menuGuid,
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
       },
     },
     {
-      name: "onItemChanged",
+      name: "bookmark-guid-changed",
       params: {
         itemId: localItemIds.get("bookmarkEEEE"),
-        property: "guid",
-        isAnnoProperty: false,
-        newValue: "bookmarkEEEE",
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
-        parentId: PlacesUtils.bookmarksMenuFolderId,
+        urlHref: "",
         guid: "bookmarkEEEE",
         parentGuid: PlacesUtils.bookmarks.menuGuid,
-        oldValue: "bookmarkEEE1",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
+        isTagging: false,
       },
     },
     {
       name: "bookmark-added",
       params: {
         itemId: localItemIds.get("bookmarkFFFF"),
-        parentId: PlacesUtils.bookmarksMenuFolderId,
+        parentId: localItemIds.get(PlacesUtils.bookmarks.menuGuid),
         index: 1,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         urlHref: "http://example.com/f",
@@ -475,12 +469,10 @@ add_task(async function test_apply_then_revert() {
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("bookmarkEEEE"),
-        oldParentId: PlacesUtils.bookmarksMenuFolderId,
         oldIndex: 2,
-        newParentId: PlacesUtils.bookmarksMenuFolderId,
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "bookmarkEEEE",
@@ -488,31 +480,29 @@ add_task(async function test_apply_then_revert() {
         newParentGuid: PlacesUtils.bookmarks.menuGuid,
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "http://example.com/e",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("folderAAAAAA"),
-        oldParentId: PlacesUtils.bookmarksMenuFolderId,
         oldIndex: 0,
-        newParentId: PlacesUtils.toolbarFolderId,
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_FOLDER,
         guid: "folderAAAAAA",
         oldParentGuid: PlacesUtils.bookmarks.menuGuid,
         newParentGuid: PlacesUtils.bookmarks.toolbarGuid,
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
-        urlHref: null,
+        urlHref: "",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("bookmarkCCCC"),
-        oldParentId: localItemIds.get("folderAAAAAA"),
         oldIndex: 1,
-        newParentId: localItemIds.get("folderAAAAAA"),
         newIndex: 0,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "bookmarkCCCC",
@@ -520,15 +510,14 @@ add_task(async function test_apply_then_revert() {
         newParentGuid: "folderAAAAAA",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "http://example.com/c",
+        isTagging: false,
       },
     },
     {
-      name: "onItemMoved",
+      name: "bookmark-moved",
       params: {
         itemId: localItemIds.get("bookmarkBBBB"),
-        oldParentId: localItemIds.get("folderAAAAAA"),
         oldIndex: 0,
-        newParentId: localItemIds.get("folderAAAAAA"),
         newIndex: 1,
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
         guid: "bookmarkBBBB",
@@ -536,36 +525,28 @@ add_task(async function test_apply_then_revert() {
         newParentGuid: "folderAAAAAA",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
         urlHref: "http://example.com/b-remote",
+        isTagging: false,
       },
     },
     {
-      name: "onItemChanged",
+      name: "bookmark-title-changed",
       params: {
         itemId: localItemIds.get("folderAAAAAA"),
-        property: "title",
-        isAnnoProperty: false,
-        newValue: "A (remote)",
-        type: PlacesUtils.bookmarks.TYPE_FOLDER,
-        parentId: PlacesUtils.toolbarFolderId,
+        title: "A (remote)",
         guid: "folderAAAAAA",
         parentGuid: PlacesUtils.bookmarks.toolbarGuid,
-        oldValue: "A",
-        source: PlacesUtils.bookmarks.SOURCES.SYNC,
       },
     },
     {
-      name: "onItemChanged",
+      name: "bookmark-url-changed",
       params: {
         itemId: localItemIds.get("bookmarkBBBB"),
-        property: "uri",
-        isAnnoProperty: false,
-        newValue: "http://example.com/b-remote",
         type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
-        parentId: localItemIds.get("folderAAAAAA"),
+        urlHref: "http://example.com/b-remote",
         guid: "bookmarkBBBB",
         parentGuid: "folderAAAAAA",
-        oldValue: "http://example.com/b",
         source: PlacesUtils.bookmarks.SOURCES.SYNC,
+        isTagging: false,
       },
     },
   ]);
@@ -647,6 +628,9 @@ add_task(async function test_apply_then_revert() {
     "Should apply new structure, second time"
   );
 
+  await storeChangesInMirror(buf, secondTimeRecords);
+  deepEqual(await buf.fetchUnmergedGuids(), [], "Should merge all items");
+
   await buf.finalize();
   await PlacesUtils.bookmarks.eraseEverything();
   await PlacesSyncUtils.bookmarks.reset();
