@@ -7,23 +7,27 @@
 
 // Trivial worker definition
 
+/* import-globals-from /toolkit/components/workerloader/require.js */
 importScripts("resource://gre/modules/workers/require.js");
 var PromiseWorker = require("resource://gre/modules/workers/PromiseWorker.js");
 
 var worker = new PromiseWorker.AbstractWorker();
-worker.dispatch = function(method, args = []) {
+worker.dispatch = function (method, args = []) {
   return Agent[method](...args);
 };
-worker.postMessage = function(...args) {
+worker.postMessage = function (...args) {
   self.postMessage(...args);
 };
-worker.close = function() {
+worker.close = function () {
   self.close();
 };
-worker.log = function(...args) {
+worker.log = function (...args) {
   dump("Worker: " + args.join(" ") + "\n");
 };
 self.addEventListener("message", msg => worker.handleMessage(msg));
+self.addEventListener("unhandledrejection", function (error) {
+  throw error.reason;
+});
 
 var Agent = {
   bounce(...args) {
